<?php

/********************************************************************
 * 
 * 管理ツールページ
 * 
 ********************************************************************/

//共通処理呼び出し
$require_login = true; //ログイン必要
require_once("./common.php");
if ($DEMO_MODE) {
    echo "デモ版では管理ツールは使用できません";
    return;
}

$command =  filter_input(INPUT_GET, 'command') ?? "";
switch ($command) {
    case 'dbbackup':
        backupSettings();
        exit;
    case 'export':
        csvExport($db);
        exit;
    case 'import':
        csvImport($db, "articles", $files['file']['tmp_name']);
        exit;
}

//設定等のバックアップアーカイブを作成してダウンロードする
function backupSettings()
{
    $zip = new ZipArchive();
    $zipFileName = './{$system_db_file}.zip';
    $dirPath = dirname(__FILE__) . '/env';

    if ($zip->open($zipFileName, ZipArchive::CREATE) === TRUE) {
        // 空のフォルダを追加
        $zip->addEmptyDir('ppp_backup');

        // フォルダ内のファイルを追加
        $files = new RecursiveIteratorIterator(
            new RecursiveDirectoryIterator($dirPath),
            RecursiveIteratorIterator::LEAVES_ONLY
        );
        foreach ($files as $name => $file) {
            if (!$file->isDir()) {
                $filePath = $file->getRealPath();
                $relativePath = substr($filePath, strlen($dirPath) + 1);
                $zip->addFile($filePath, 'ppp_backup/' . $relativePath);
            }
        }
        $zip->close();

        // ファイルをダウンロード
        $date = new DateTime('now', new DateTimeZone('Asia/Tokyo'));
        $yyyymmdd = $date->format('Ymd');
        header('Content-Type: application/zip');
        header('Content-Disposition: attachment; filename="ppp_backup_' . $yyyymmdd . '.zip"');
        readfile($zipFileName);

        // ダウンロード後、ファイルを削除
        unlink($zipFileName);
    } else {
        echo 'アーカイブの作成に失敗しました。';
    }
}

//CSV出力
function csvExport($db, $tableName = "articles")
{
    $outputCsvFile = './env/' . bin2hex(random_bytes(rand(15, 20))) . '.csv';
    $file = fopen($outputCsvFile, 'w');

    $query = "PRAGMA table_info($tableName)";
    $result = $db->query($query);
    $headers = [];
    while ($row = $result->fetchArray(SQLITE3_ASSOC)) {
        $headers[] = $row['name'];
    }
    fputcsv($file, $headers);

    $query = "SELECT * FROM $tableName";
    $result = $db->query($query);
    while ($row = $result->fetchArray(SQLITE3_ASSOC)) {
        fputcsv($file, $row);
    }

    fclose($file);

    // ファイルをダウンロード
    $date = new DateTime('now', new DateTimeZone('Asia/Tokyo'));
    $yyyymmdd = $date->format('Ymd');
    header('Content-Type: text/csv');
    header('Content-Disposition: attachment; filename="ppp_export_' . $tableName . "_" . $yyyymmdd . '.csv"');
    readfile($outputCsvFile);

    // ダウンロード後、ファイルを削除
    unlink($outputCsvFile);
}

//CSV入力
function csvImport($db, $tableName, $importCsvFile)
{
    // ファイルを開く
    $file = fopen($importCsvFile, 'r');
    $count = 1;
    try {
        // テーブルのカラム情報を取得
        $query = "PRAGMA table_info($tableName)";
        $result = $db->query($query);
        // カラム名の配列を初期化
        $columns = [];

        while ($row = $result->fetchArray(SQLITE3_ASSOC)) {
            $columns[] = $row['name'];
        }
        $db->exec('begin Immediate');
        // ヘッダーを無視する場合はコメントアウトを解除
        $header = fgetcsv($file);
        $columnsList = implode(",", $header);

        // データを読み込み、テーブルに挿入または更新
        while (($row = fgetcsv($file)) !== false) {
            // カラムの列挙
            //$columnsList = implode(", ", $columns);

            // 値の設定
            $valuesList = "'" . implode("', '", $row) . "'";

            // INSERT OR REPLACE文を実行
            $query = "INSERT OR REPLACE INTO $tableName ($columnsList) VALUES ($valuesList)";
            $result = $db->exec($query);

            // エラーハンドリング
            if (!$result) {
                throw new Exception("データベース挿入/更新エラー: " . $db->lastErrorMsg() . "<br>" . $count . "行目でエラーが発生しました");
            }
        }
        $db->exec('commit');
        echo '{"status":"error", "message":"CSVインポートが完了しました"}';
    } catch (Exception $e) {
        $db->exec('callback');
        echo '{"status":"error", "message":"CSVインポートに失敗しました<br>' . $e->getMessage() . '"}';
    } finally {
        // ファイルを閉じる
        fclose($file);
    }
}

?>
<!doctype html>
<html lang="ja">

<head>
    <meta charset="utf-8">
    <title>管理ツール</title>
    <meta name="description" content="">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <meta name="theme-color" content="#fafafa">
    <link rel="stylesheet" href="./style/recss.css">
    <link rel="stylesheet" href="./style/base.css">
    <link rel="icon" href="./resource/icon.png">
    <meta http-equiv="Pragma" content="no-cache">
    <meta http-equiv="Cache-Control" content="no-cache">
    <link rel="stylesheet" href="./style/system.css">
    <style>
        #control {
            margin-bottom: 2em;
            display: flex;
            width: 100%;
            flex-direction: row;
            flex-wrap: wrap
        }

        .button {
            width: 40%;
            margin: 1em auto 1em auto;
        }

        .tool {
            margin-bottom: 3em;

        }

        #update-warn {
            font-size: 0.8em;
            color: red;
        }
    </style>
    <link rel="stylesheet" href="./module/fontawesome/css/all.min.css" rel="preload">
    <script src="https://ajax.googleapis.com/ajax/libs/jquery/3.7.1/jquery.min.js"></script>
    <script src="./script/dialog.js?ver=<?= $VERSION ?>"></script>
    <link rel="stylesheet" href="./style/dialog.css">
    <script src="./script/console.js?ver=<?= $VERSION ?>"></script>

    <script>
        //非同期バージョンチェック
        async function versionCheck() {
            return new Promise((resolve, reject) => {
                $.ajax({
                    type: "GET",
                    url: "./api/version.php",
                    success: function(response) {
                        console.log("Success: " + response);
                        var result = JSON.parse(response);
                        resolve(result)
                    },
                    error: function() {
                        reject();
                    }
                });
            });
        }

        $(async function() {
            result = await versionCheck();

            $("#update-msg").html("[ " + result["message"] + " ]");
            if (result["new-version"] == "") {
                //最新バージョン利用中

            } else {
                //旧バージョンのためアップデートを促す
                $("#version-up-button").removeClass("button_disable");
                $("#version-up-button").on("click", async function(e) {
                    e.preventDefault();
                    ret = await $("body").showDialog("<p>バージョンアップを実行します。<br>※アップデート前にバックアップを取ってください。<br>よろしいですか？</p>", "OK", "キャンセル");
                    if (ret == "OK") {
                        location.href = './api/version.php?command=update';
                    }
                });
            }



        })
    </script>
</head>

<body>
    <div id="header">
        <a id="back-link" href="./setting.php" class="header-link-exit">&lt; 戻る</a>
        <div id="header-option-link-area">
            <a id="logout-link" href="./logout.php">ログアウト</a>
        </div>
    </div>
    <div id="base">
        <div id="control">


            <div id="contents-area">
                <div class="tool">
                    <h3>バージョンアップ</h3>
                    <p>
                        PPPを最新のバージョンにアップデートします。<br>
                        <span id="update-msg">[バージョン確認中]</span><br>
                        <span id="update-warn">※アップデート実施前にかならシステムバックアップを行ってください</span>
                    </p>
                    <button id="version-up-button" class="button button_disable">アップデート</button>
                </div>
                <div class="tool">
                    <h3>システムバックアップ</h3>
                    <p>
                        データベース・設定値等をダウンロードしてバックアップします。<br>
                        ※画像はバックアップされません。<br>
                    </p>
                    <button id="backup-button" class="button" onClick="location.href='?command=dbbackup'">ダウンロード</button>
                </div>
                <div class="tool">
                    <h3>CSV出力(記事情報)</h3>
                    <p>
                        全記事の本文データをCSV出力します。<br>
                    </p>
                    <button id="backup-button" class="button" onClick="location.href='?command=export'">ダウンロード</button>
                </div>
                <div class="tool">
                    <h3>システムテンプレート管理</h3>
                    <p>
                        PPPの動作で使用されるテンプレートを管理します<br>
                    </p>
                    <button id="backup-button" class="button" onClick="location.href='./template_console.php'">テンプレート管理画面</button>
                </div>
                <div class="tool">
                    <h3>外部連携設定</h3>
                    <p>
                        外部からの書き込みを許可する連携設定を行います。<br>
                    </p>
                    <button id="backup-button" class="button" onClick="location.href='./external_token_console.php'">外部連携管理画面</button>
                </div>
                <div class="tool">
                    <h3>ポータル連携設定</h3>
                    <p>
                        PPPPortalとの連携設定を行います。<br>
                    </p>
                    <button id="backup-button" class="button" onClick="location.href='./portal_console.php'">ポータル連携管理画面</button>
                </div>
            </div>
        </div>
    </div>



</body>

</html>