<?php

//記事投稿関連クラス
class Article
{
    private $DB;
    private $SETTING;
    private $DEMO_MODE;
    function __construct($db, $setting, $demo)
    {
        $this->DB = $db;
        $this->SETTING = $setting;
        $this->DEMO_MODE = $demo;
    }

    public function Sort($post_mode)
    {
        $post_mode = mb_strtolower($post_mode);

        //POST内容を取得
        $forms = $this->filterInput($post_mode);

        //GuestModeCheck
        if (!$this->GuestKeyCheck($forms)) {
            throw new Exception('{"status":"error", "message":"削除キーが無効です"}');
        }

        try {
            $this->DB->exec('begin Immediate');

            $idx = 0;
            $sorts = explode(";", $forms["sortList"]); // ;区切りでソートしたい順にファイル名が渡される
            foreach ($sorts as $sort) {
                $stmt = $this->DB->prepare('update images set sort = :sort Where file_name = :fileName');
                $stmt->bindValue(':sort',  $idx + 1);
                $stmt->bindValue(':fileName',  $sort);
                $stmt->execute();
                $idx++;
            }

            $this->DB->exec('commit');

            return ['id' => $forms["id"], 'image_index' => $idx];
        } catch (Exception $ex) {
            $this->DB->exec('rollback');
            throw $ex;
        }
    }
    public function Post()
    {
        //POST内容を取得
        $forms = $this->filterInput('');

        //内容チェック
        if (!$forms['created_at'] && !$forms['comment']) {
            //日付もコメントもない場合はエラー
            throw new Exception('パラメータが不正です');
        }
        if ($forms['alt_url']) {
            $alt_url = $forms['alt_url'];
            $check  = strpos($alt_url, '/');
            if ($check === 0) {
                throw new Exception('"/"から始まる固定URLは指定できません');
            }
            $check  = strpos($alt_url, 'api/');
            if ($check === 0) {
                throw new Exception('"api/"から始まる固定URLは指定できません');
            }
            if ($alt_url == "log") {
                throw new Exception('"logは固定URLとして指定できません".');
            }
            if (preg_match('/^log\/\d+(\/.*)?$/i', $alt_url)) {
                throw new Exception('"log/(数字)の固定URLは指定できません".');
            }

            if (file_exists($alt_url)) {
                throw new Exception('この固定URLは指定できません');
            }
        }


        try {
            $this->DB->exec('begin Immediate');

            $id = $this->DBWrite($forms, $forms['id']);

            //パスワード設定されている記事 or エントリーパスワードが設定されている場合はファイル名にランダム要素を付与する
            $rnd = false;

            $last_image_idx = $this->imagePost($id, $_FILES, $forms['sortList'], $rnd)['index'];
            $this->RemoveMedia($id, $forms['removeimages']);

            $this->image_comment_write($id, $forms['image-comments']);

            $this->DB->exec('commit');

            return ['id' => $id, 'image_index' => $last_image_idx, 'alt_url' => $forms['alt_url']];
        } catch (Exception $ex) {
            $this->DB->exec('rollback');
            throw $ex;
        }
    }
    public function ExternalPost()
    {
        //POST内容を取得
        $forms = $this->filterInput('external');

        if (!$forms['created_at'] && !$forms['comment']) {
            //日付もコメントもない場合はエラー
            throw new Exception('パラメータが不正です');
        }

        try {
            $this->DB->exec('begin Immediate');

            $id = $this->DBWrite($forms, $forms['id']);
            $last_image_idx = $this->imagePost($id, $_FILES, $forms['sortList'], false)['index'];
            $this->RemoveMedia($id, $forms['removeimages']);

            $this->DB->exec('commit');

            return ['id' => $id, 'image_index' => $last_image_idx];
        } catch (Exception $ex) {
            $this->DB->exec('rollback');
            throw $ex;
        }
    }
    public function GuestPost()
    {
        //POST内容を取得
        $forms = $this->filterInput('guest');

        if (!$forms['created_at'] && !$forms['comment']) {
            //日付もコメントもない場合はエラー
            throw new Exception('パラメータが不正です');
        }

        //GuestModeCheck
        if (!$this->GuestKeyCheck($forms)) {
            throw new Exception('{"status":"error", "message":"削除キーが無効です"}');
        }

        try {
            $this->DB->exec('begin Immediate');

            $id = $this->DBWrite($forms, $forms['id']);
            $last_image_idx = $this->imagePost($id, $_FILES, $forms['sortList'], false)['index'];
            $this->RemoveMedia($id, $forms['removeimages']);

            $this->DB->exec('commit');
        } catch (Exception $ex) {
            $this->DB->exec('rollback');
            throw $ex;
        }


        return ['id' => $id, 'image_index' => $last_image_idx];
    }

    //編集モードで削除キーが指定されている場合、有効チェックする
    //新規モード、削除キーの指定がない場合は通過する
    function GuestKeyCheck($forms)
    {

        if ($forms['id'] && $forms['delete_key']) {
            $stmt = $this->DB->prepare("select id,delete_key From articles where id=:id and delete_key not null and delete_key != ''");
            $stmt->bindValue(':id', $forms['id']);
            $result = $stmt->execute();
            $row = $result->fetchArray();
            if (!$row || !password_verify($forms['delete_key'], $row['delete_key'])) {
                return false;
            }
        }
        return true;
    }

    //指定の記事および関連するメディアファイルを削除する
    function RemoveArticle($id)
    {
        $this->DB->exec('begin Immediate');
        //ファイルの削除
        $stmt = $this->DB->prepare('SELECT * FROM images Where article_id= :id');
        $stmt->bindValue(':id', $id);
        $result = $stmt->execute();
        while ($row = $result->fetchArray()) {
            if (file_exists("../images/{$row["file_name"]}")) {
                unlink("../images/{$row["file_name"]}");
            }
            if (file_exists("../images/{$row["thumb_name"]}")) {
                unlink("../images/{$row["thumb_name"]}");
            }
            if (file_exists("../images/{$row["thumb_l_name"]}")) {
                unlink("../images/{$row["thumb_l_name"]}");
            }
        }
        //ログの更新
        $stmt = $this->DB->prepare("delete From articles where id=:id");
        $stmt->bindValue(':id',  $id);
        $result = $stmt->execute();

        $stmt = $this->DB->prepare("delete from images where article_id=:id");
        $stmt->bindValue(':id',  $id);
        $result = $stmt->execute();
        $this->DB->exec('commit;');
    }

    //入力値取得
    //$post_mode:
    //  "" or "normal" : 標準
    //  "external" : 外部投稿時
    //  "guest" : ゲスト投稿時
    function FilterInput($post_mode)
    {
        $post_mode = mb_strtolower($post_mode);

        $forms = array();
        $forms['mode'] = filter_input(INPUT_POST, 'mode') ?? "post";

        $forms['id'] = filter_input(INPUT_POST, 'id', FILTER_SANITIZE_NUMBER_INT);
        $forms['thumbnail'] = filter_input(INPUT_POST, 'thumbnail');
        $forms['created_at'] =  filter_input(INPUT_POST, 'created_at');
        $forms['title'] =  filter_input(INPUT_POST, 'title') ?? "";
        $forms['comment'] =  filter_input(INPUT_POST, 'comment') ?? "";
        $forms['comment'] = base64_decode($forms['comment']); //WAF対策
        $forms['alt_url'] = filter_input(INPUT_POST, 'alt_url') ?? "";
        $forms['alt_url'] = rtrim($forms['alt_url'], '/');
        $forms['tags'] =  filter_input(INPUT_POST, 'tags') ?? "";
        $forms['tags'] = str_replace("　", " ", $forms['tags']); //全角スペースを半角に
        if ($post_mode == "guest") {
            if (!strstr($forms['tags'], "ゲスト記事")) {
                $forms['tags'] = $forms['tags'] . " ゲスト記事";
            }
        }
        $forms['tags'] = " " . $forms['tags'] . " "; //前後に空欄を付与（完全一致検索用）


        $forms['pin'] =  filter_input(INPUT_POST, 'pin') ?? false;
        $forms['sort'] =  filter_input(INPUT_POST, 'pin_sort', FILTER_SANITIZE_NUMBER_INT) ?? 1;
        if ($forms['pin'] == false) {
            //ピン止めされていない記事の優先度は常に1
            $forms['sort'] = "1";
        }

        $forms['hidden'] =  filter_input(INPUT_POST, 'hidden') ?? false;
        $forms['nsfw'] =  filter_input(INPUT_POST, 'nsfw') ?? false;
        $forms['use_alt_ogimage'] =  filter_input(INPUT_POST, 'use_alt_ogimage') ?? false;

        $forms['style'] =  filter_input(INPUT_POST, 'style') ?? 0;
        $forms['use_html'] =  true;
        $forms['show_password'] = filter_input(INPUT_POST, 'show_password') ?? '';
        $forms['image-comments'] = $_POST['image-comments'] ?? [];
        switch ($forms['style']) {
            case 1: //詳細記事
                $forms['use_full_html'] = true;
                break;
            case 99: //固定ページ
                $forms['use_full_html'] = true;
                if (!$forms['id']) {
                    $forms['hidden'] = true;
                }
                break;
            default:
                $forms['use_full_html'] = false;
                break;
        }

        $forms['sortList'] =  filter_input(INPUT_POST, 'sort') ?? "";
        $forms['removeimages'] = filter_input(INPUT_POST, "removeimages") ?? "";

        //日時指定がない場合は現在日時を使用
        if (!$forms['created_at']) {
            date_default_timezone_set('Asia/Tokyo'); // タイムゾーンを設定
            $now = date('Y-m-d H:i:s');
            $forms['created_at'] = $now; // 日時をフォーマット
        }

        //ExternalPost・GuestPost
        $forms['delete_key'] = filter_input(INPUT_POST, 'delete_key') ?? "";
        $forms['from_ip'] = $_SERVER["REMOTE_ADDR"] ?? "";

        //DEMO版対応
        if ($this->DEMO_MODE) {
            //リンク・スクリプトコードを取り除く
            $forms['comment'] = preg_replace('/<\?/i', "", $forms['comment']);
            $forms['comment'] = preg_replace('/\?>/i', "", $forms['comment']);
            $forms['comment'] = preg_replace('/<.*?a .*?>/i', "", $forms['comment']);
            $forms['comment'] = preg_replace('/<.*?link .*?>/i', "", $forms['comment']);
            $forms['comment'] = preg_replace('/<.*?script.*?>/i', "", $forms['comment']);
            $forms['comment'] = preg_replace('/<.*?iframe.*?>/i', "", $forms['comment']);
            $forms['comment'] = preg_replace('/action=/i', "", $forms['comment']);
            $forms['comment'] = preg_replace('/rel=/i', "", $forms['comment']);
            $forms['comment'] = preg_replace('/{link:.*?}/i', "", $forms['comment']);
        }

        //モード別挙動
        if ($post_mode == "guest") {
            //ゲストモードで削除キーなしは不許可
            if ($forms['delete_key'] == "") {
                throw new Exception('削除キーが指定されていません');
            }

            //ゲストモードの場合HTMLを制限
            $forms['comment'] =  strip_tags($forms['comment']); //HTMLタグ除去
            $forms['comment'] =  preg_replace('/{([0-9]?)}/i', " }$1{ ", $forms['comment']); //画像リンク以外の特殊タグを削除
            $forms['comment'] =  preg_replace('/{.*?}/i', "", $forms['comment']);
            $forms['comment'] =  preg_replace('/ }[0-9]*?{ /i', "", $forms['comment']);
            $forms['use_html'] =  false; //HTML不許可
            $forms['use_full_html'] =  false; //通常記事固定
        } else if ($post_mode == "external") {
            //外部投稿の場合HTMLを制限
            if ($forms['delete_key'] == "") {
                $forms['use_html'] =  false; //HTML不許可
            }
            $forms['use_full_html'] =  false; //通常記事固定
        }
        return $forms;
    }

    function DBWrite($forms, $id = "")
    {
        //記事　記録
        if ($id == "") {
            //新規
            if ($forms['delete_key'] != "") {
                $stmt = $this->DB->prepare("INSERT INTO articles (title, comment, tags, created_at, updated_at, pin, sort, hidden, nsfw, use_alt_ogimage, use_html, use_full_html, style, delete_key, from_ip, alt_url, thumbnail, show_password) VALUES (:title, :comment, :tags, :created_at, :updated_at, :pin, :sort, :hidden, :nsfw, :use_alt_ogimage, :use_html, :use_full_html, :style, :delete_key, :from_ip, :alt_url, :thumbnail, :show_password)");
            } else {
                $stmt = $this->DB->prepare("INSERT INTO articles (title, comment, tags, created_at, updated_at, pin, sort, hidden, nsfw, use_alt_ogimage, use_html, use_full_html, style, from_ip, alt_url, thumbnail, show_password) VALUES (:title, :comment, :tags, :created_at, :updated_at, :pin, :sort, :hidden, :nsfw, :use_alt_ogimage, :use_html, :use_full_html, :style, :from_ip, :alt_url, :thumbnail, :show_password)");
            }
        } else {
            //編集
            if ($forms['delete_key'] != "") {
                $stmt = $this->DB->prepare("Update articles set title=:title, comment=:comment, tags=:tags, created_at=:created_at, updated_at=:updated_at, pin=:pin, sort=:sort, hidden=:hidden, nsfw = :nsfw, use_alt_ogimage = :use_alt_ogimage, use_html = :use_html, use_full_html = :use_full_html, style = :style, delete_key = :delete_key, from_ip = :from_ip , alt_url = :alt_url , thumbnail = :thumbnail, show_password = :show_password where id=:id");
            } else {
                $stmt = $this->DB->prepare("Update articles set title=:title, comment=:comment, tags=:tags, created_at=:created_at, updated_at=:updated_at, pin=:pin, sort=:sort, hidden=:hidden, nsfw = :nsfw, use_alt_ogimage = :use_alt_ogimage, use_html = :use_html, use_full_html = :use_full_html, style = :style, from_ip = :from_ip  , alt_url = :alt_url , thumbnail = :thumbnail, show_password = :show_password  where id=:id");
            }
        }
        $stmt->bindValue(':id', $id);
        date_default_timezone_set('Asia/Tokyo'); // タイムゾーンを設定
        $now = date('Y-m-d H:i:s');
        $stmt->bindValue(':updated_at', $now);

        $stmt->bindValue(':title', $forms['title']);
        $stmt->bindValue(':comment', $forms['comment']);
        $stmt->bindValue(':created_at', $forms['created_at']);
        $stmt->bindValue(':use_html', $forms['use_html'] ? 1 : 0);
        $stmt->bindValue(':use_full_html', $forms['use_full_html'] ? 1 : 0);
        $stmt->bindValue(':thumbnail', $forms['thumbnail']);
        $stmt->bindValue(':style', $forms['style']);
        $stmt->bindValue(':tags', $forms['tags']);
        $stmt->bindValue(':alt_url', $forms['alt_url']);
        $stmt->bindValue(':pin', $forms['pin'] ? 1 : 0);
        $stmt->bindValue(':sort', $forms['sort']);
        $stmt->bindValue(':hidden', $forms['hidden'] ? 1 : 0);
        $stmt->bindValue(':nsfw', $forms['nsfw'] ? 1 : 0);
        $stmt->bindValue(':use_alt_ogimage', $forms['use_alt_ogimage'] ? 1 : 0);
        $stmt->bindValue(':show_password', $forms['show_password']);
        if ($forms['delete_key'] != "") {
            $stmt->bindValue(':delete_key', password_hash($forms['delete_key'], PASSWORD_DEFAULT));
        }
        $stmt->bindValue(':from_ip ', $forms['from_ip']);
        $stmt->execute();

        if ($id == "") {
            $id = $this->DB->lastInsertRowID();
        }

        return $id;
    }

    function image_comment_write($article_id, $comments)
    {
        $index = 1;
        foreach ($comments as $comment) {
            $stmt = $this->DB->prepare("Update images set comment = :comment  where article_id=:article_id and sort=:sort");
            $stmt->bindValue(':article_id', $article_id);
            $stmt->bindValue(':sort', $index++);
            $stmt->bindValue(':comment', $comment);
            $stmt->execute();
        }
    }
    function ImageCommentPost($article_id, $file_name, $comment)
    {
        $stmt = $this->DB->prepare("Update images set comment = :comment  where article_id=:article_id and file_name=:file_name");
        $stmt->bindValue(':article_id', $article_id);
        $stmt->bindValue(':file_name', $file_name);
        $stmt->bindValue(':comment', $comment);
        $stmt->execute();
    }

    //メディアファイルの投稿
    //画像ファイル：<input type="file" name="images[]">を対象とする
    //動画ファイル：<input type="file" name="moviews[]">を対象とする。<input type="file" name="movie_wpreviews[]">をプレビュー画像として使用。
    function ImagePost($article_id, $files, $sortList, $use_random)
    {
        //該当ログに紐づいている画像一覧から一番大きいimage_idを取得し、+1したものを新規IDとする
        $image_idx = $this->DB->querySingle('SELECT MAX(image_id) AS maxSort FROM images WHERE article_id = ' . $article_id) + 1;
        if (!isset($image_idx)) {
            $image_idx = 1;
        }
        $sort_idx = $this->DB->querySingle('SELECT MAX(sort) AS maxSort FROM images WHERE article_id = ' . $article_id) + 1;
        if (!isset($sort_idx)) {
            $sort_idx = 1;
        }

        //リサイズ処理用モジュール読み込み
        require_once('imageEdit.php');
        $newFileNames = "";
        //画像アップロード
        if (isset($files['images'])) {
            foreach ($files['images']['tmp_name'] as $index => $tmpName) {
                if ($use_random) {
                    //ファイル名にランダム性を持たせる
                    $rnd = "_" . bin2hex(random_bytes(20));
                } else {
                    $rnd = "";
                }

                $fileName = $files['images']['name'][$index];
                $fileSize = $files['images']['size'][$index];
                $fileName = preg_replace('/\.jpeg$/i', '.jpg', $fileName); //jpegをjpgに変更
                $extension = substr($fileName, strrpos($fileName, '.') + 1);
                $newFileName = "image_" . $article_id . "_" . $image_idx . $rnd . "." .  strtolower($extension);
                $thumbFileName = "image_" . $article_id . "_" . $image_idx . $rnd . ".jpg"; //サムネイルはJPG固定
                $sortList = str_replace($fileName, $newFileName, $sortList);

                //リサイズ
                $imginfo = createResizeImage($tmpName, $this->SETTING['resize_border'], $this->SETTING['resize_border'], "../images/$newFileName", 90);

                //大サムネイル作成
                $thumb_l_info = createResizeImage($tmpName, 900, 0, "../images/thumb_l_$thumbFileName", 80);

                //小サムネイル作成
                $thumb_info = createResizeImage($tmpName, 300, 300, "../images/thumb_$thumbFileName", 80);

                $stmt = $this->DB->prepare("INSERT INTO images (article_id, image_id, sort, file_name, width, height, thumb_l_name, thumb_l_width, thumb_l_height, thumb_name, thumb_width, thumb_height ) " .
                    "VALUES (:article_id, :image_id, :sort, :newFileName, :width, :height, :thumb_l_name, :thumb_l_width, :thumb_l_height,:thumb_name, :thumb_width, :thumb_height)");
                $stmt->bindValue(':article_id', $article_id);
                $stmt->bindValue(':image_id', $image_idx);
                $stmt->bindValue(':sort', $sort_idx);
                $stmt->bindValue(':newFileName', $newFileName);
                $stmt->bindValue(':width', $imginfo['width']);
                $stmt->bindValue(':height', $imginfo['height']);
                $stmt->bindValue(':newFileName', $newFileName);
                $stmt->bindValue(':width', $imginfo['width']);
                $stmt->bindValue(':height', $imginfo['height']);
                $stmt->bindValue(':thumb_l_name', 'thumb_l_' . $thumbFileName);
                $stmt->bindValue(':thumb_l_width', $thumb_l_info['width']);
                $stmt->bindValue(':thumb_l_height', $thumb_l_info['height']);
                $stmt->bindValue(':thumb_name', 'thumb_' . $thumbFileName);
                $stmt->bindValue(':thumb_width', $thumb_info['width']);
                $stmt->bindValue(':thumb_height', $thumb_info['height']);
                $stmt->execute();

                $sort_idx++;
                $image_idx++;
                $newFileNames .= $newFileName . ";";
            }
        }
        //動画アップロード
        if (!$this->DEMO_MODE && isset($files['movies'])) {
            foreach ($files['movies']['tmp_name'] as $index => $tmpName) {
                $preview_fileName = $files['movie_previews']['name'][$index];
                $preview_fileSize = $files['movie_previews']['size'][$index];
                $preview_tmpName = $files['movie_previews']['tmp_name'][$index];
                $preview_fileName = preg_replace('/\.jpeg$/i', '.jpg', $preview_fileName); //jpegをjpgに変更
                $movie_fileName = $files['movies']['name'][$index];
                $extension = substr($files['movies']['name'][$index], strrpos($files['movies']['name'][$index], '.') + 1);
                $newFileName = "image_" . $article_id . "_" . $image_idx . "." .  strtolower($extension);
                $thumbFileName = "image_" . $article_id . "_" . $image_idx . ".jpg"; //サムネイルはJPG固定
                $sortList = str_replace($movie_fileName, $newFileName, $sortList);

                //動画はそのままコピー
                copy($tmpName, "../images/$newFileName");

                //大サムネイル作成
                $thumb_l_info = createResizeImage($preview_tmpName, 900, 0, "../images/thumb_l_$thumbFileName", 80);

                //小サムネイル作成
                $thumb_info = createResizeImage($preview_tmpName, 300, 300, "../images/thumb_$thumbFileName", 80);

                $stmt = $this->DB->prepare("INSERT INTO images (article_id, image_id, sort, file_name, width, height, thumb_l_name, thumb_l_width, thumb_l_height, thumb_name, thumb_width, thumb_height, type ) " .
                    "VALUES (:article_id, :image_id, :sort, :newFileName, :width, :height, :thumb_l_name, :thumb_l_width, :thumb_l_height,:thumb_name, :thumb_width, :thumb_height, :type)");
                $stmt->bindValue(':article_id', $article_id);
                $stmt->bindValue(':image_id', $image_idx);
                $stmt->bindValue(':sort', $sort_idx);
                $stmt->bindValue(':newFileName', $newFileName);
                $stmt->bindValue(':width', $thumb_l_info['width']);
                $stmt->bindValue(':height', $thumb_l_info['height']);
                $stmt->bindValue(':newFileName', $newFileName);
                $stmt->bindValue(':thumb_l_name', 'thumb_l_' . $thumbFileName);
                $stmt->bindValue(':thumb_l_width', $thumb_l_info['width']);
                $stmt->bindValue(':thumb_l_height', $thumb_l_info['height']);
                $stmt->bindValue(':thumb_name', 'thumb_' . $thumbFileName);
                $stmt->bindValue(':thumb_width', $thumb_info['width']);
                $stmt->bindValue(':thumb_height', $thumb_info['height']);
                $stmt->bindValue(':type', 'video');
                $stmt->execute();

                $sort_idx++;
                $image_idx++;
                $newFileNames .= $newFileName . ";";
            }
        }

        //ソート順の更新
        $idx = 1;
        $sorts = explode(";", $sortList);
        foreach ($sorts as $sort) {
            $stmt = $this->DB->prepare('update images set sort = :sort Where file_name = :fileName');
            $stmt->bindValue(':sort',  $idx);
            $stmt->bindValue(':fileName',  $sort);
            $stmt->execute();
            $idx++;
        }

        return ["index" => $image_idx, "file_names" => $newFileNames];
    }

    //メディアファイルの削除
    function RemoveMedia($id, $removeMedias)
    {
        foreach (explode(' ', $removeMedias) as $removeId) {
            if ($removeId != "") {
                $stmt = $this->DB->prepare('SELECT * FROM images WHERE article_id = :id AND id = :imageId');
                $stmt->bindValue(':id',  $id);
                $stmt->bindValue(':imageId',  $removeId);
                $result = $stmt->execute();

                $row = $result->fetchArray();
                if ($row) {
                    if (file_exists("../images/{$row["file_name"]}")) {
                        unlink("../images/{$row["file_name"]}");
                    }
                    if (file_exists("../images/{$row["thumb_name"]}")) {
                        unlink("../images/{$row["thumb_name"]}");
                    }
                    if (file_exists("../images/{$row["thumb_l_name"]}")) {
                        unlink("../images/{$row["thumb_l_name"]}");
                    }

                    $stmt = $this->DB->prepare('DELETE FROM images WHERE article_id = :id AND id = :imageId');
                    $stmt->bindValue(':id', $id);
                    $stmt->bindValue(':imageId', $removeId);
                    $result = $stmt->execute();
                }
            }
        }
    }

    //指定された記事の、サムネイル対象の画像を返す。
    //imageテーブルのRowを配列として返す。
    public function GetThmbnailImage($target_id)
    {
        try {

            $stmt = $this->DB->prepare('SELECT * From images Where article_id= :id Order by sort ASC  LIMIT 1');
            $stmt->bindValue(':id',  $target_id);
            $result = $stmt->execute();
            $row = $result->fetchArray();
            if ($row) {
                return $row;
            } else {
                return null;
            }
        } catch (Exception $ex) {
            return null;
        }
    }
}
