<?php

//共通処理呼び出し
$require_login = true; //ログイン必要
require_once("./common.php");

$command = filter_input(INPUT_GET, 'command') ?? "";
if ($command == "reset") {
    //リセット
    $file = './resource/default_user.css';
    $css = file_get_contents($file);
    echo $css;
    exit;
}

$content = filter_input(INPUT_POST, 'css');
if ($content) {
    $content = base64_decode($content); //WAF対策
    if ($content) {
        $file = './env/user.css';
        $css = $content;
        file_put_contents($file, $content);

        $sql = "UPDATE setting SET css_update = :css_update";
        $stmt = $db->prepare($sql);
        date_default_timezone_set('Asia/Tokyo'); // タイムゾーンを設定
        $datetime = date("YmdHis"); // 日時をフォーマット
        $stmt->bindValue(':css_update',  $datetime ?? "");
        $result = $stmt->execute();


        echo '{"status":"success"}';
        exit;
    }
}
$file = './env/user.css';
$css = file_get_contents($file);
?>


<!doctype html>
<html lang="ja">

<head>
    <meta charset="utf-8">

    <meta http-equiv="Pragma" content="no-cache">
    <meta http-equiv="Cache-Control" content="no-cache">

    <title>デザイン編集</title>
    <meta name="description" content="">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <meta name="theme-color" content="#fafafa">
    <link rel="stylesheet" href="./style/recss.css">
    <link rel="stylesheet" href="./style/base.css">
    <link rel="stylesheet" href="./style/system.css">
    <link rel="stylesheet" href="https://ajax.googleapis.com/ajax/libs/jqueryui/1.13.2/themes/smoothness/jquery-ui.css">
    <link rel="stylesheet" href="./module/fontawesome/css/all.min.css" rel="preload">
    <link rel="icon" href="./resource/icon.png">
    <script src="https://ajax.googleapis.com/ajax/libs/jquery/3.7.1/jquery.min.js"></script>
    <script src="https://ajax.googleapis.com/ajax/libs/jqueryui/1.13.2/jquery-ui.min.js"></script>
    <script src="./script/dialog.js?ver=<?= $VERSION ?>"></script>
    <link rel="stylesheet" href="./style/dialog.css">
    <script src="./script/tooltip.js?ver=<?= $VERSION ?>"></script>
    <style>
        #css {
            height: 80vh;
        }

        #submit-area {
            margin-top: 2em;
            text-align: center;
        }

        #reset-area {
            margin-top: 3em;
            text-align: center;
        }
    </style>
    <script>
        //非同期送信処理本体
        async function postContents(form_data) {
            return new Promise((resolve, reject) => {
                $.ajax({
                    type: "POST",
                    url: './design_edit.php',
                    data: form_data,
                    processData: false,
                    contentType: false,
                    success: function(response) {
                        console.log('Success: ' + response);
                        let result = JSON.parse(response);
                        resolve(result)
                    },
                    error: function() {
                        reject();
                    }
                });
            });
        }
        async function resetContents() {
            return new Promise((resolve, reject) => {
                $.ajax({
                    type: "GET",
                    url: './design_edit.php',
                    data: {
                        "command": "reset"
                    },
                    success: function(response) {
                        resolve(response);
                    },
                    error: function() {
                        reject();
                    }
                });
            });
        }

        function setUnloadCheck() {
            $(window).off('beforeunload');
            $(window).on('beforeunload', async function() {
                return "ページを移動しようとしています。\n入力した内容が失われますがよろしいですか？";
            });
        }

        function unsetUnloadCheck() {
            $(window).off('beforeunload');
        }

        $(async function() {
            $("form input, form textarea, form select").change(function() {
                setUnloadCheck();
            });

            //投稿・更新ボタンクリック
            $("#submit").on('click', async function(e) {
                e.preventDefault();
                let wait = $("body").waitDialog('<img src="./resource/loading.gif"><p>処理中…</p>');

                //記事本体送信
                let form_data = new FormData($("form")[0]);
                //WAF対策のため本文をBASE64エンコード
                form_data.set('css', btoa(unescape(encodeURIComponent($('#css').val()))));

                let postResult = "";
                try {
                    postResult = await postContents(form_data);
                } catch {
                    wait.close();

                    //投稿失敗
                    let dialogText = '<p>' + msgtitle + 'に失敗しました</p>';
                    $("body").showDialog(dialogText, 'OK');

                    return;
                }
                wait.close();
                let dialogText = '<p>更新しました</p>';
                unsetUnloadCheck();
                $("body").showDialog(dialogText, 'OK');
            });

            $("#reset").on('click', async function(e) {
                e.preventDefault();
                ret = await $('body').showDialog("<p>デザインをリセットします。<br>変更は失われます。<br><br>よろしいですか？<br><br></p>", "OK", "キャンセル");
                if (ret == "OK") {
                    css = await resetContents();
                    $('#css').val(css);
                    setUnloadCheck();
                }

            });


            //Ctrl + Sショートカット
            $(document).keydown(function(e) {
                if (e.ctrlKey && (e.which == 83)) {
                    e.preventDefault();
                    $('#submit').click();
                }
            });
        });
    </script>
</head>

<body>
    <div id="header">
        <a id="back-link" href="./setting.php" class="header-link-exit">
            &lt; 戻る
        </a>
        <div id="header-option-link-area">
            <a id="logout-link" href="./logout.php">ログアウト</a>
        </div>
    </div>
    <div id="base">
        <div id="contents-area">
            <form action="./design_edit.php" method="POST">
                <div class="input-area" id="input-area" class="flat-input__noborder">
                    <textarea name="css" id="css" class="flat-input__noborder" placeholder="CSS"><?= $css ?></textarea>
                </div>
                <div id="submit-area">
                    <button id="submit" name="submit" class="button button_submit">保存</button>
                </div>
                <div id="reset-area">
                    <button id="reset" class="button button_warn" data-target="<?= $target ?>">初期化</button>
                </div>
            </form>
        </div>
        <footer id="footer-area">
        </footer>
    </div>
</body>

</html>