<?php

/********************************************************************
 * 
 * 編集ページ
 *    GET   id:編集対象のLOG ID。この値がない場合は新規投稿モード。
 * 
 ********************************************************************/

//共通処理呼び出し
$require_login = true; //ログイン必要
require_once("./common.php");
refreshToken("./"); //ログイントークンをリフレッシュ

//アップロード可能なファイルサイズを得る
$max_send_size = getMaxUploadSize();

//入力値の受け取り
$target = filter_input(INPUT_GET, 'id', FILTER_SANITIZE_NUMBER_INT); //処理対象LOGID

$mode = "post";
$page_title = "記事投稿";
$submit_title = "投稿";
$postmode_show = "";
$editmode_show = "hide";
$previews = "";
if ($target) {
  //処理対象のLOGIDが指定されている場合は更新モード
  $mode = "edit";
  $postmode_show = "hide";
  $editmode_show = "";
  $page_title = "記事更新";
  $submit_title = "更新";
}

require_once('./env/tagcloud.php');


//Shereテンプレート構築
$share_template = "[";
$result = $db->query("select * from templates where template_name = 'share_link' order by template_sort asc");
while ($row = $result->fetchArray(SQLITE3_ASSOC)) {
  $share_template .= '{"title":"' .  $row['value']  . '","action":"' . $row['option1'] . $row['option2'] . '"},';
}
$share_template .= "]";

//EXIFテンプレート構築
$exif_template = "";
if ($SETTING['write_exif']) {
  $result = $db->query("select * from templates where template_name = 'exif' order by template_sort asc");
  while ($row = $result->fetchArray(SQLITE3_ASSOC)) {
    $exif_template .=  $row['value'] ?? "";
  }
}


if ($mode == "edit") {
  //編集モードのため、対象ログを取得する
  try {
    $stmt = $db->prepare('SELECT * From articles Where id= :id');
    $stmt->bindValue(':id',  $target);
    $result = $stmt->execute();
    if ($row = $result->fetchArray()) {
      $title = h($row['title'] ?? "");
      $comment = h($row['comment'] ?? "");
      $tags = h(trim($row['tags']) ?? "");
      $created_at = h($row['created_at'] ?? "");
      $pin = $row['pin'] ? "checked" : "";
      $sort = $row['sort'] ?? "";
      $nsfw = $row['nsfw'] ? "checked" : "";
      $hidden = $row['hidden']  ? "checked" : "";
      $views = $row['views'];
      $like = $row['like'];
      $use_alt_ogimage = $row['use_alt_ogimage'] ? "checked" : "";
      $alt_url = h($row['alt_url'] ?? "");
      $use_html = $row['use_html'] ? "checked" : "";
      $use_full_html = $row['use_full_html'] ? "checked" : "";
      $article_style = $row['style'] ?? 0;
      $thumbnail = $row['thumbnail'] ?? "";
      $show_password = $row['show_password'] ?? "";
    } else {
      //指定されたログが存在しない
      $db->close();
      header('Location: console.php');
      exit;
    }

    //画像一覧構築
    $stmt = $db->prepare('SELECT * FROM images Where article_id= :id Order by sort ASC');
    $stmt->bindValue(':id',  $target);
    $result = $stmt->execute();
    while ($row = $result->fetchArray()) {
      $d = "'";
      $previews = $previews . '<li class="preview-item" data-filename="' . h($row["file_name"])  .  '" ondblclick="editImageComment(event)"><img class="preview-item-image" id="preview-' . h($row["file_name"])  . '" src="./images/' . h($row["thumb_l_name"])  . '" >' .
        '<button type="button" onClick="$(this).parent().remove();addRemoveImageList(' . $d . $row["id"] . $d . ')">×</button><i class="fa-regular fa-circle-right mover" style="display:none"></i><i class="fa-regular fa-circle-left movel"  style="display:none"></i><input type="hidden" class="image-comment"  name="image-comments[]" value="' . h($row["comment"]) .  '"></li>';
    }
  } catch (Exception $ex) {
    echo "システムエラーが発生しました";
    exit;
  } finally {
    $db->close();
  }
} else {
  //新規モードのため全て空にする
  $comment = $SETTING["contents_template"] ?? "";
  $tags = "";
  $use_html = "";
  $use_full_html = "";
  $nsfw = "";
  $pin = "";
  $use_user_thumbnail = "";
  $hidden = "";
  $views = "-";
  $like = "-";
  $alt_url = "";
  $sort = "1";
  date_default_timezone_set('Asia/Tokyo'); // タイムゾーンを設定
  $created_at = date('Y-m-d H:i:s'); // 日時をフォーマット
  $now_date = date("Y-m-d");
  $now_y = date("Y");
  $now_m = date("m");
  $now_d = date("d");
  $now_time = date('H:i:s');
  $show_password = "";
  $use_alt_ogimage = 0;
  $thumbnail = "";
  $article_style = $SETTING["default_style"] ?? 0;

  $title = str_replace("%date%", $now_date, str_replace("%datetime%", $created_at, $SETTING["title_template"] ?? ""));
  $title = str_replace("%yyyy%", $now_y, $title);
  $title = str_replace("%mm%", $now_m, $title);
  $title = str_replace("%dd%", $now_d, $title);
  $title = str_replace("%time%", $now_time, $title);
}

$thumb_files = getFileList("./env/thumbnails/");
$thumb_list = "";
foreach ($thumb_files as $thumb_item) {
  $selected = "";
  if ($thumbnail == $thumb_item) {
    $selected = " selected";
  }
  $thumb_list .= '<option value="' . $thumb_item . '"' .  $selected . '>' . $thumb_item . '</option>';
}

function getFileList($dir)
{
  $files = [];
  if (is_dir($dir)) {
    $files = scandir($dir);
    $files = array_diff($files, array('.', '..', 'index.html', '.htaccess'));
  }
  return $files;
}

?>

<!doctype html>
<html lang="ja">

<head>
  <meta charset="utf-8">

  <meta http-equiv="Pragma" content="no-cache">
  <meta http-equiv="Cache-Control" content="no-cache">

  <title><?= $page_title ?></title>
  <meta name="description" content="">
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <meta name="theme-color" content="#fafafa">
  <link rel="stylesheet" href="./style/recss.css">
  <link rel="stylesheet" href="./style/base.css">
  <link rel="stylesheet" href="./style/system.css">
  <link rel="stylesheet" href="./style/edit.css">
  <link rel="stylesheet" href="https://ajax.googleapis.com/ajax/libs/jqueryui/1.13.2/themes/smoothness/jquery-ui.css">

  <link rel="stylesheet" href="./module/fontawesome/css/all.min.css" rel="preload">
  <link rel="icon" href="./resource/icon.png">
  <script src="https://ajax.googleapis.com/ajax/libs/jquery/3.7.1/jquery.min.js"></script>
  <script src="https://ajax.googleapis.com/ajax/libs/jqueryui/1.13.2/jquery-ui.min.js"></script>
  <script src="https://cdnjs.cloudflare.com/ajax/libs/exif-js/2.3.0/exif.min.js"></script>
  <script src="./script/edit.js?ver=<?= $VERSION ?>"></script>
  <script>
    //クライアントリサイズのボーダーを設定
    max_size = <?= $SETTING['resize_border'] ?>;
  </script>

  <script src="./script/dialog.js?ver=<?= $VERSION ?>"></script>
  <link rel="stylesheet" href="./style/dialog.css">
  <script src="./script/tooltip.js?ver=<?= $VERSION ?>"></script>
</head>

<body>
  <div id="header">
    <a id="back-link" href="./console.php" class="header-link-exit">
      &lt; 戻る
    </a>
    <div id="header-option-link-area">
      <?php if ($mode == "edit") { ?>
        <a id="view-link" href="./view.php?id=<?= $target ?>">閲覧ページを開く</a>
      <?php } ?>
      <a id="logout-link" href="./logout.php">ログアウト</a>
    </div>
  </div>
  <div id="base">
    <div id="contents-area">
      <form action="article_post.php" method="post" enctype="multipart/form-data" id="inputForm">
        <input type="file" name="tmp" id="fileSelect" accept="image/*,.mp4" multiple class="hide" data-maxsize="<?= $max_send_size ?>">
        <input type="text" name="id" id="id" class="hide" value="<?= $target; ?>">
        <input type="text" name="removeimages" class="hide" id="removeimages">
        <input type="text" name="sort" class="hide" id="sort">
        <div>
          <div id="dropArea" class="dropArea <?= $postmode_show ?>">
            <span style="display:inline-block;margin-top:14%"> 画像ファイルをドラッグ<br />または<br />タップで画像を選択</span>
          </div>
          <div id="previewArea" class="<?= $editmode_show ?>">
            <ul id="previewList">
              <?= ($previews) ?>
              <li id="dropArea2Root">
                <div id="dropArea2" class="dropArea">
                  <span style="display:inline-block;margin-top:14%"> 画像ファイルをドラッグ<br />または<br />タップで画像を選択</span>
                </div>
              </li>
            </ul>
          </div>
        </div>
        <div class="thumb-setting">
          サムネイル：
          <select class="flat-input" id="thumbnail" name="thumbnail">
            <option value="" <?php echo $thumbnail == "" ? "selected" : "" ?>>自動</option>
            <option value="@all" <?php echo $thumbnail == "@all" ? "selected" : "" ?>>自動（全画像をサムネイル表示する）</option>
            <option value="@user_thumbnail" <?php echo $thumbnail == "@user_thumbnail" ? "selected" : "" ?>>1枚目の画像をサムネイルとして扱う</option>
            <option value="@text" <?php echo $thumbnail == "@text" ? "selected" : "" ?>>サムネイルなし</option>
            <?= $thumb_list ?>
          </select>
        </div>
        <div class="view-info-area"><?php if ($SETTING['use_feedback']) { ?><span class='like-it'><i class='fa-regular fa-eye'></i> <?= $views ?> <i id='like_heart' class='fa-regular fa-heart'></i> <?= $like ?></span><?php } ?></div>
        <div class="input-area" id="input-area" class="flat-input__noborder">
          <input type="text" id="created_at" name="created_at" value="<?= $created_at ?>" placeholder="作成日時"><i class="fa-solid fa-rotate-right" id="time_reload"></i><br>
          <input type="text" id="title" name="title" value="<?= $title ?>" placeholder="タイトル">
          <hr />
          <textarea name="comment" id="comment" class="flat-input__noborder" placeholder="コメント" wrap="off"><?= $comment ?></textarea>
        </div>
        <div style="margin-top:0.5em">
          スタイル：
          <select name="style" class="flat-input">
            <option value="0" <?php echo $article_style == 0 ? "selected" : "" ?>>シンプル</option>
            <option value="1" <?php echo $article_style == 1 ? "selected" : "" ?>>詳細記事</option>
            <option value="4" <?php echo $article_style == 4 ? "selected" : "" ?>>漫画[左]</option>
            <option value="5" <?php echo $article_style == 4 ? "selected" : "" ?>>漫画[右]</option>
            <option value="2" <?php echo $article_style == 2 ? "selected" : "" ?>>小説[横]</option>
            <option value="3" <?php echo $article_style == 3 ? "selected" : "" ?>>小説[縦]</option>
            <option value="99" <?php echo $article_style == 99 ? "selected" : "" ?>>固定ページ</option>
          </select>
          <a href="https://ppp.kannagi.net/view.php?id=2" style="font-weight:100; font-size:0.9em;" target="_blank">[ヘルプ]</a> <a href="#" id="show_command_palette" style="font-weight:100; font-size:0.9em;" target="_blank">[コマンド一覧]</a>
        </div>
        <div class="command_palette hide">
          <div class="command_palette_inner">
            <ul>
              <li class="insert_item">{caption} {/caption}</li>
              <li class="insert_item">{1}</li>
              <li class="insert_item">{2}</li>
              <li class="insert_item">{3}</li>
              <li class="insert_item">{c}</li>
              <li class="insert_item">{search search_word}</li>
              <li class="insert_item">{link index_or_url text}</li>
              <li class="insert_item">{article_list tag sort style count}</li>
              <li class="insert_item">{mailform}</li>
              <li class="insert_item">{vt} {/vt}</li>
              <li class="insert_item">{tc} {/tc}</li>
              <li class="insert_item">{rb}漢字|カナ{/rb}</li>
              <li class="insert_item">{border}</li>
              <li class="insert_item">{border #color}</li>
              <li class="insert_item">{hidden target}</li>
              <li class="insert_item">{md} {/md}</li>
              <li class="insert_item">{more} {/more}</li>
            </ul>
          </div>

          <div>
            <button class="button_normal button command_palette_close">閉じる</button>
          </div>
        </div>
        <div class="image_palette hide">
          <div class="image_palette_inner">
            <ul>
              <li class="insert_item">{caption} {/caption}</li>
              <li class="insert_item">{1}</li>
              <li class="insert_item">{2}</li>
              <li class="insert_item">{3}</li>
              <li class="insert_item">{c}</li>
              <li class="insert_item">{search search_word}</li>
              <li class="insert_item">{link index_or_url text}</li>
              <li class="insert_item">{article_list tag sort style count}</li>
              <li class="insert_item">{mailform}</li>
              <li class="insert_item">{vt} {/vt}</li>
              <li class="insert_item">{tc} {/tc}</li>
              <li class="insert_item">{rb}漢字|カナ{/rb}</li>
              <li class="insert_item">{border}</li>
              <li class="insert_item">{border #color}</li>
              <li class="insert_item">{hidden target}</li>
              <li class="insert_item">{md} {/md}</li>
              <li class="insert_item">{more} {/more}</li>
            </ul>
          </div>

          <div>
            <button class="button_normal button image_palette_close">閉じる</button>
          </div>
        </div>
        <div class="input-area" id="tag-input-area">
          <input type="text" id="tags" name="tags" value="<?= $tags; ?>" placeholder="タグ">
        </div>
        <span style="font-size:0.8em;color:#555">タグはスペース区切りで入力してください。タグそのものにスペースは使用できません。タグ先頭に「-」を付けると隠しタグになります。</span>
        <div id="tagcloud">
          <?= $tagcloud_nolink_sys ?? $tagcloud_nolink ?>
          <?= $tagcloud_nolink_more_sys ?? $tagcloud_links_more ?>
        </div>

        <div id="options-area">
          <input type="checkbox" name="pin" id="pin" <?= $pin ?>><label for="pin">トップページにピン止めする</label><span style="font-size:0.8em">( 優先度:<input type="text" name="pin_sort" class="flat-iput" style="border:1px solid #999; width:3em;padding-left:0.5em" value="<?= $sort ?>"> )</span><br>
          <input type="checkbox" name="nsfw" id="nsfw" <?= $nsfw ?>><label for="nsfw">NSFW(R18)</label><br>
          <input type="checkbox" name="use_alt_ogimage" id="use_alt_ogimage" <?= $use_alt_ogimage ?>><label for="use_alt_ogimage">TwitterCardに画像を表示しない</label><br>
          <input type="checkbox" name="hidden" id="hidden" <?= $hidden ?>><label for="hidden">一覧に表示しない(下書きなど)</label><br><br>
          <!-- 表示用あいことば : <input type="input" name="show_password" id="show_password" value="<?= $show_password ?>" class="flat-input"><br><br> -->
        </div>
        <div id="url-area">
          固定URL <input type="text" id="alt_url" name="alt_url" class="flat-input" style="width:100%" value="<?= $alt_url ?>" placeholder="特別な固定URLを設定したい場合に指定します"><br>
          <span style="font-size:0.8em;color:#555">特定の固定URLを設定したい場合に指定します。 <?= $PATH_INFO["url_system_root"] ?>/****の****部分を指定します。</span>
        </div>
        <div id="submit-area">
          <button id="submit" name="submit" class="button button_submit"><?= $submit_title ?></button>
        </div>
      </form>
      <?php if ($mode == "edit") { ?>
        <div id="delete-area">
          <button id="delete" class="button button_warn" data-target="<?= $target ?>">削除</button>
        </div>

      <?php } ?>
    </div>
    <footer id="footer-area">
    </footer>
    <div class="sp-fix-footer">
      <button id="submit2" name="submit2" class="button button_submit"><?= $submit_title ?></button>
    </div>
  </div>
  <script>
    $mode = '<?= $mode ?>';
    $tweetmode = '<?php echo $SETTING["tweet_service_key"] ? "api" : "normal" ?>';
    $copy_clipboard = '<?php echo $SETTING["tweet_use_clipboard"] ? true : false ?>';
    $share_template = <?php echo $share_template ?>;
    $exif_template = '<?php echo $exif_template ?>';
  </script>
  <span style="display:none" class="sitetitle"><?= $SETTING["site_title"] ?></span>
</body>

</html>