<?php

/********************************************************************
 * 
 * トップページ
 *    GET   q   :  検索ワード。スペース区切りで複数指定するとAND検索。
 *    GET   page:  ページネーションインデックス。
 * 
 ********************************************************************/
ini_set('display_errors', "On");
//共通処理呼び出し
$require_login = false; //ログイン不要
require_once("./common.php");

//入力値の受け取り
$searchs =  filter_input(INPUT_GET, 'q');
$index =  filter_input(INPUT_GET, 'page', FILTER_VALIDATE_INT) ?? 1;
$static_mode = filter_input(INPUT_GET, 'static_mode', FILTER_SANITIZE_NUMBER_INT) ?? 0; //静的出力モード

//1ページ当たり記事数
$records_per_page = $SETTING["index_articles"] ?? 28;
$limit = $records_per_page;
$offset = 0;
if ($index) {
    $offset = ($index - 1)  * $records_per_page;
}

try {
    //アカウント設定の読み取り
    $account_rows = $db->query('SELECT * FROM accounts');
    while ($row = $account_rows->fetchArray()) {
        if (!strstr($row['url'], "http")) {
            $row['url'] = "mailto:" . $row['url'];
        }
        $accounts[] = $row;
    }

    //メニュー読み取り
    $menus = [];
    $home_menu = "";
    $result = $db->query('SELECT id, created_at, title, comment, tags, nsfw, alt_url From articles' .
        ' Where style=99 ORDER BY created_at DESC');
    while ($row = $result->fetchArray()) {
        if ($SETTING["use_rewrite"]) {
            if ($row['alt_url']) {
                $url = "./{$row['alt_url']}";
            } else {
                $url = "./log/{$row['id']}";
            }
        } else {
            $url = "./view.php?id={$row['id']}";
        }

        $home_menu = $home_menu . "<a href='$url' class='home_menu_item_a'><li  class='home_menu_item' data-id='{$row['id']}' data-url='$url'>{$row['title']}</li></a>";
        $menus[] = ['id' => $row['id'], 'url' =>  $url, 'title' => $row['title'], 'is_current' =>  0];
    }
    if ($home_menu != "") {
        $home_menu = '<ul class="home_menu"><li class="home_menu_item home_menu_item_current" data-id="home">HOME</li>' . $home_menu . '</ul>';
    }

    //非表示タグ一覧
    $exclude_index_list_tag = '';
    if ($SETTING['exclude_index_list_tag']) {
        foreach (explode(' ', $SETTING['exclude_index_list_tag']) as $tag) {
            $exclude_index_list_tag = $exclude_index_list_tag .  " and tags not like '%" . h($tag) . "%'";
        }
    }

    //記事一覧生成
    $article_count = 0;
    if ($searchs == '') {
        //検索なし
        //hidden指定されていない記事を$records_per_page分取得する
        $article_count = $db->querySingle('SELECT Count(id) as count From articles WHERE hidden != 1 ' . $exclude_index_list_tag);
        $pages = ceil($article_count / $records_per_page);

        $result = $db->query('SELECT * From articles' .
            ' Where hidden != 1 ' . $exclude_index_list_tag . 'ORDER BY PIN Desc, sort DESC, created_at DESC LIMIT ' .
            $records_per_page . ' OFFSET ' . $offset);
    } else {
        //検索条件あり
        //検索条件に従って記事を検索する。スペース区切りで複数指定があった場合はAND条件とする。
        //検索条件式の構築
        $search_list = '';
        foreach (explode(' ', $searchs) as $search_Idx => $search) {
            $search_list = $search_list .  " and (title like :search{$search_Idx} or comment like :search{$search_Idx} or tags like :search{$search_Idx})";
        }
        //検索結果数の取得
        $stmt = $db->prepare('SELECT Count(id) as count From articles WHERE hidden != 1 AND ' . $search_list . ' ' . $exclude_index_list_tag);
        foreach (explode(' ', $searchs) as $search_Idx => $search) {
            $stmt->bindValue(":search{$search_Idx}",  '%' . $search . '%');
        }
        $result = $stmt->execute();
        $row = $result->fetchArray(); //結果は常に1行
        $article_count = $row['count'];
        $pages = ceil($article_count / $records_per_page);

        //記事の取得
        $stmt = $db->prepare('SELECT * From articles WHERE hidden != 1 ' . $search_list .  ' ' . $exclude_index_list_tag .
            ' ORDER BY pin DESC, sort DESC, created_at DESC LIMIT ' . $records_per_page . ' OFFSET ' . $offset);
        foreach (explode(' ', $searchs) as $search => $search_Idx) {
            $stmt->bindValue(":search{$search_Idx}",  '%' . $search . '%');
        }
        $result = $stmt->execute();
    }

    //記事一覧の生成 ( $articles )
    $idx = 0;
    $list = '';
    $articles = array();
    if ($SETTING['exclude_index_list_tag'] != "*ALL*") {
        while ($row = $result->fetchArray()) {
            $article_id = h($row['id']);
            $html_url = "./view.php?id={$article_id}";
            $index_all_images = ($row['thumbnail'] == "@all") || $SETTING['index_all_images'];
            $articles[$idx]['id'] = h($row['id']);
            $articles[$idx]['pin'] = $row['pin'];
            if ($row['pin']) {
                $html_pin = "<span class='pin_image'> </span>";
            } else {
                $html_pin = "";
            }
            $articles[$idx]['created_at'] = h($row['created_at']);
            $articles[$idx]['updated_at'] = h($row['updated_at'] ?? "");
            $articles[$idx]['title'] = h($row['title']);
            $articles[$idx]['nsfw'] = $row['nsfw'];
            $comment = $row['comment'];
            if ($comment) {
                preg_match('/{caption}(.*?){\/caption}/si', $comment, $match);
                if ($match) {
                    //キャプションがある場合はそれを使用
                    $comment = $match[1];
                } else {
                    $comment = mb_substr(strip_tags($comment), 0, 100);
                }
                //特殊記述の削除
                $comment = preg_replace('/\{.*\}/', '', $comment);
                $comment = preg_replace('/\r\n/', ' ', $comment);
                $comment = preg_replace('/\n/', ' ', $comment);
                $comment = strip_tags($comment);

                $articles[$idx]['comment'] = $comment;
            } else {
                $articles[$idx]['comment'] = '';
            }


            if ($SETTING['use_rewrite']) {
                $articles[$idx]['alt_url'] = $row['alt_url'] ? $row['alt_url'] : "log/{$article_id}";
                $html_url = $articles[$idx]['alt_url'];
            } else {
                $articles[$idx]['alt_url'] = "";
            }

            //記事に画像添付があるか確認
            $articles[$idx]['image'] = "";
            $articles[$idx]['file_name'] = "";
            $articles[$idx]['thumb_name'] =  "";
            $articles[$idx]['thumb_height'] =  "";
            $articles[$idx]['thumb_width'] =  "";
            $articles[$idx]['thumb_l_name'] =  "";
            $articles[$idx]['thumb_l_height'] = "";
            $articles[$idx]['thumb_l_width'] =  "";
            $articles[$idx]['multi_image'] = 0;

            $copy_tmp = null;
            $articles[$idx]['image']  = "";

            if ($row["thumbnail"] == "" || $row['thumbnail']  == "@user_thumbnail" || $row['thumbnail']  == "@all") {
                $thumbs = $db->query('SELECT * FROM images WHERE article_id = ' . $article_id . ' ORDER BY sort ASC');
                $sub_index = 0;
                while ($trow = $thumbs->fetchArray()) {
                    if ($copy_tmp == null) {
                        $copy_tmp = $articles[$idx];
                    } else {
                        $articles[$idx] = $copy_tmp;
                    }
                    //サムネイルは拡張子jpg
                    $articles[$idx]['image'] = preg_replace('/\.(gif|png|mp4|webm|webp)/i', '.jpg', h($trow['file_name']));
                    $articles[$idx]['file_name'] =  h($trow['file_name']);
                    $articles[$idx]['thumb_name'] =  h($trow['thumb_name']);
                    $articles[$idx]['thumb_height'] =  h($trow['thumb_height']);
                    $articles[$idx]['thumb_width'] =  h($trow['thumb_width']);
                    $articles[$idx]['thumb_l_name'] =  h($trow['thumb_l_name']);
                    $articles[$idx]['thumb_l_height'] =  h($trow['thumb_l_height']);
                    $articles[$idx]['thumb_l_width'] =  h($trow['thumb_l_width']);
                    if (!$index_all_images) {
                        $trow = $thumbs->fetchArray(); //次の一件を取得をできるか確認して複数画像があるかチェックする
                        if ($row['thumbnail']  == "@user_thumbnail") {
                            $trow = $thumbs->fetchArray(); //1枚目がサムネイルの場合はさらにもう一枚あるかチェック
                        }
                        if ($trow) {
                            $html_multi = "<span class='multi_image'> </span>";
                            $articles[$idx]['multi_image'] = 1;
                        } else {
                            $html_multi = "";
                            $articles[$idx]['multi_image'] = 0;
                        }
                    } else {
                        $html_multi = "";
                        $articles[$idx]['multi_image'] = 0;
                    }
                    if ($row['nsfw'] && !$NSFW_CONSENT) {
                        $html_thumb = './resource/nsfw.png';
                        $html_thumb_class = "list-thumb-nsfw background-nsfw";
                        $html_thumb_background = "";
                    } else {
                        $html_thumb = "./images/{$articles[$idx]['thumb_name']}";
                        $html_thumb_class = "list-thumb-image";
                        $html_thumb_background = "style=\"background-image:url('$html_thumb')\"";
                    }
                    if ($row['pin']) {
                        $html_thumb_class .= " thumb_pin";
                    }
                    $html_thumb_title = "";

                    if ($sub_index > 0) {
                        $articles[$idx]['link'] = $html_url . '#' . ($sub_index + 1);
                        if ($articles[$idx]['alt_url']) {
                            $articles[$idx]['alt_url'] = $articles[$idx]['alt_url'] . '#' . ($sub_index + 1);
                        }
                    } else {
                        $articles[$idx]['link'] = $html_url;
                    }
                    $articles[$idx]['fix_background'] = $html_thumb_background;
                    $articles[$idx]['fix_thumb'] = $html_thumb;
                    $articles[$idx]['fix_thumb_class'] = $html_thumb_class;
                    $articles[$idx]['fix_thumb_title'] = $html_thumb_title;
                    $articles[$idx]['fix_multi_image'] = "<span class='thumb_info'>" . $html_multi .  $html_pin . "</span>";
                    $articles[$idx]['fix_thumb_html'] = "<div class='$html_thumb_class' $html_thumb_background>{$html_thumb_title}{$html_multi}</div>";

                    $sub_index++;
                    $idx++;
                    if (!$index_all_images) {
                        break;
                    }
                }
            }
            if ($copy_tmp == null) {
                //画像なし
                $articles[$idx]['link'] = $html_url;
                $articles[$idx]['image'] = "";
                $html_thumb = "";
                $html_thumb_class = "list-thumb-text";

                if ($row['nsfw'] && !$NSFW_CONSENT) {
                    $title = '<span class="nsfw-titile">[NSWF]</span>' . $articles[$idx]['title'];
                } else {
                    $title = $articles[$idx]['title'];
                }
                $html_thumb_title = "<div class='list-thumb-text-inner'>$title</div>";
                if ($row['thumbnail'] != "" && $row['thumbnail'] != "@text" && $row['thumbnail'] != "@user_thumbnail" && $row['thumbnail'] != "@all") {
                    $html_thumb = "./env/thumbnails/{$row['thumbnail']}";
                    $html_thumb_background = "style=\"background-image:url('$html_thumb')\"";
                } else {
                    $html_thumb_background = "";
                }
                $html_multi = "";

                $articles[$idx]['fix_background'] = $html_thumb_background;
                $articles[$idx]['fix_thumb'] = $html_thumb;
                $articles[$idx]['fix_thumb_class'] = $html_thumb_class;
                $articles[$idx]['fix_thumb_title'] = $html_thumb_title;
                if ($row['pin']) {
                    $articles[$idx]['fix_multi_image'] = "<span class='thumb_info'>" .  $html_pin . "</span>";
                    $html_thumb_class .= " thumb_pin";
                } else {
                    $articles[$idx]['fix_multi_image'] = "";
                }
                $articles[$idx]['fix_thumb_html'] = "<div class='$html_thumb_class' $html_thumb_background>{$html_thumb_title}{$html_multi}</div>";
                $idx++;
            }
        }
    }
    //タグクラウド用データ読み取り
    require_once('./env/tagcloud.php');

    //サイト説明文内の特殊構文を変換
    $instruction_text = $SETTING["instruction_text"];

    //最新記事情報の取り出し
    $first_article_comment = "";
    $first_article_title = "";
    $first_article_image = "";
    $first_article_image_s = "";
    $first_article_id = "";
    $first_article_image_html = "";
    $first_article_image_s_html = "";
    if (strpos($instruction_text, '{new-')) {
        //説明文に{new-系統タグが含まれている場合は最新の記事を取得
        $result = $db->query('SELECT id, created_at, title, comment, tags, nsfw, alt_url From articles' .
            ' Where hidden != 1 ORDER BY created_at DESC LIMIT 1');
        if ($row = $result->fetchArray()) {
            $first_article_comment = mb_substr(strip_tags($row['comment']), 0, 300);
            $first_article_title = h($row['title']);
            $first_article_id =  h($row['id']);
            if ($SETTING['use_rewrite']) {
                $url = h($row['alt_url'] ? $row['alt_url'] : "log/" . $first_article_id);
            } else {
                $url = h($row['alt_url'] ? $row['alt_url'] : "view.php?id=" . $first_article_id);
            }
            //記事に画像添付があるか確認
            $thumbs = $db->query('SELECT * FROM images WHERE article_id = ' . $first_article_id . ' ORDER BY sort ASC');
            $trow = $thumbs->fetchArray(); //最初の1件を取得
            if ($trow) {
                //画像ありの記事
                if (!$SETTING["nsfw"] && $row['nsfw'] && !$NSFW_CONSENT) {
                    //NSFW確認が取れていない
                    $first_article_image = "./resource/nsfw.png";
                    $first_article_image = '<a href="./' . $url . '"><img class="instraction-image" src="./resource/nsfw.png"></a>';
                    $first_article_image_s = '<a href="./' . $url . '"><img class="instraction-image" src="./resource/nsfw.png"></a>';
                } else {
                    $first_article_image  = h($trow['thumb_l_name']);
                    $first_article_image_html = '<a href="./' . $url . '"><img class="instraction-image" src="./images/' . $first_article_image . '" width=' . h($trow['thumb_l_width']) . ' height=' . h($trow['thumb_l_height']) . '></a>';
                    $first_article_image_s_html = '<a href="./' . $url . '"><img class="instraction-image" src="./images/' .  h($trow['thumb_name']) . '" width=' . h($trow['thumb_width']) . ' height=' . h($trow['thumb_height']) . '></a>';
                }
            } else {
                //画像なしの記事
                $first_article_image = "";
            }
        }
        if ($first_article_image != "") {
            $instruction_text = preg_replace('/{new-thumb}/i', $first_article_image_html, $instruction_text);
            $instruction_text = preg_replace('/{new-thumb-s}/i', $first_article_image_s_html, $instruction_text);
        } else {
            $instruction_text = preg_replace('/{new-thumb}/i', '', $instruction_text);
            $instruction_text = preg_replace('/{new-thumb-s}/i', '', $instruction_text);
        }
        $instruction_text = preg_replace('/{new-title}/i', $first_article_title, $instruction_text);
        $instruction_text = preg_replace('/{new-comment}/i', $first_article_comment, $instruction_text);
    }

    //記事内画像参照
    $instruction_text = preg_replace('/{(\d+)\-(\d+)}/i', "<img class='instraction-image' src='./images/thumb_l_image_$1_$2.jpg'>", $instruction_text);
    $instruction_text = preg_replace('/{(\d+)\-(\d+) s}/i', "<img class='instraction-image' src='./images/thumb_image_$1_$2.jpg'>", $instruction_text);
    $instruction_text = preg_replace('/{(\d+)\-(\d+) (\d+) (\d+)}/i', "<img class='instraction-image' src='./images/thumb_l_image_$1_$2.jpg' width='$3' height='$4'>", $instruction_text);
    $instruction_text = preg_replace('/{(\d+)\-(\d+) s (\d+) (\d+)}/i', "<img class='instraction-image' src='./images/thumb_image_$1_$2.jpg' width='$3' height='$4'>", $instruction_text);
    $instruction_text = preg_replace('/{(\d+)\-(\d+) t}/i', "<a href='./view.php?id=$1'><div class=\"list-thumb-image\" style=\"background-image:url('./images/thumb_image_$1_1.jpg')\"></div><div class='list-info'></div></a>", $instruction_text);

    //リンク系統
    $instruction_text = preg_replace('/{link ([0-9]*?)}(.*?){\/link}/i', '<a href="./view.php?id=$1">$2</a>', $instruction_text);
    $instruction_text = preg_replace('/{link http(.*?)}(.*?){\/link}/i', '<a href="http$1" target="_blank">$2</a>', $instruction_text);
    $instruction_text = preg_replace('/{link (.*?)}(.*?){\/link}/i', '<a href="./view.php?id=$1">$2</a>', $instruction_text);
    $instruction_text = preg_replace('/{link:([0-9]*?)}.*?{\/link}/i', '<a href="./view.php?id=$1">$2</a>', $instruction_text);
    $instruction_text = preg_replace('/{link:http(.*?)}(.*?){\/link}/i', '<a href="http$1" target="_blank">$2</a>', $instruction_text);
    $instruction_text = preg_replace('/{link:(.*?)}(.*?){\/link}/i', '<a href="./view.php?id=$1">$2</a>', $instruction_text);

    $instruction_text = preg_replace('/{search:(.*?)}/i', '<a href="./list.php?q=$1">$1</a>', $instruction_text);
    $instruction_text = preg_replace('/{tag:(.*?)}/i', '<a href="./list.php?q=%23$1" class="tag">#$1</a>', $instruction_text);
    $instruction_text = preg_replace('/{tag (.*?)}/i', '<a href="./list.php?q=%23$1" class="tag">#$1</a>', $instruction_text);
    $instruction_text = preg_replace('/{link:([0-9]*?) (.*?)}/i', '<a href="./view.php?id=$1">$2</a>', $instruction_text);
    $instruction_text = preg_replace('/{link:http(.*?) (.*?)}/i', '<a href="http$1" target="_blank">$2</a>', $instruction_text);
    $instruction_text = preg_replace('/{link:http(.*?)}/i', '<a href="http$1" target="_blank">http$1</a>', $instruction_text);
    $instruction_text = preg_replace('/{link:(.*?) (.*?)}/i', '<a href="./view.php?id=$1">$2</a>', $instruction_text);
    $instruction_text = preg_replace('/{search (.*?)}/i', '<a href="./list.php?q=$1">$1</a>', $instruction_text);
    $instruction_text = preg_replace('/{link ([0-9]*?) (.*?)}/i', '<a href="./view.php?id=$1">$2</a>', $instruction_text);
    $instruction_text = preg_replace('/{link http(.*?) (.*?)}/i', '<a href="http$1" target="_blank">$2</a>', $instruction_text);
    $instruction_text = preg_replace('/{link http(.*?)}/i', '<a href="http$1" target="_blank">http$1</a>', $instruction_text);
    $instruction_text = preg_replace('/{link (.*?) (.*?)}/i', '<a href="./view.php?id=$1">$2</a>', $instruction_text);

    //その他本文用データ構築 ( $document )
    $document = array();
    $doucment['menus'] = $menus;
    $document['nsfw_consent'] = $NSFW_CONSENT;
    $document['max_index'] = $pages;
    $document['min_index'] = 1;
    $document['current_index'] = $index;
    $document['article_count'] = $article_count;
    $document['search_words'] = $searchs;
    if ($SETTING['use_tag_cloud']) {
        $document['tag_cloud'] = $tagcloud_links . $tagcloud_links_more;
    } else {
        $document['tag_cloud'] = "";
    }
    $document['instruction_text'] = $instruction_text;

    //Twitterカード用データ
    $document['card_title'] = $SETTING['site_title'];
    $document['card_comment'] = $SETTING['twitter_card_text'];
    $document['card_thumbnail'] = $PATH_INFO['url_system_root'] . '/images/banner.jpg';
    $document['card_ogimage']    = $PATH_INFO['url_system_root'] . '/images/ogimage.jpg';
} catch (Exception $ex) {
    echo ("システムエラーが発生しました" . $ex->getMessage());
    exit;
} finally {
    $db->close();
}
//UIレンダリング
require_once('./module/NoSixTemplate.php');
if (file_exists('./env/templates/template_index.php')) {
    //ユーザーテンプレートがある場合はそちらを採用する
    $template = new NoSixTemplate('template_index.php', './env/templates/');
} else {
    $template = new NoSixTemplate('template_index.php', './templates');
}
$template->set_data('setting', $SETTING); //Array
$template->set_data('accounts', $accounts); //Array[](url, icon)
$template->set_data('articles', $articles); //Array[](id,title,created_at,comment,image)
$template->set_data('home_menu', $home_menu);
$template->set_data('document', $document); //Array(max_index,min_index,current_index,article_count,search_words)
$template->set_data('login', $LOGIN); //ログイン有無
$template->set_Data('static_mode', $static_mode);
$template->set_Data('VERSION', $VERSION);

if ($static_mode == 1) {
    //静出力モード
    $file_name = "./index.html";
    ob_start();

    $template->display();

    $output = ob_get_contents();
    ob_end_clean();
    file_put_contents($file_name, $output);
    echo ('{"state":"success", "path", "' . $file_name . '"}');
    exit;
}

$template->display();
