<?php

/************************************************************************************** 
 * initialize.php
 * 本システム利用開始時の初期化処理を行います
 *
 * env ディレクトリ内の「system_setting.php」「user_setting.php」の存在を確認して
 * 処理モードが変更されます。
 *
 *「system_setting.php」が存在しない場合：完全初期化モード
 *「user_setting.php」が存在しない場合  ：ユーザーID・パスワード変更モード
 *
 ***************************************************************************************/

ini_set('display_errors', "On");

$SYSTEM_SETTING_FILE = './env/system_setting.php';
$USER_SETTING_FILE = './env/user_setting.php';

$DEMO_USER = "";
$DEMO_PASSWORD = "";

$message = "";
//HTTPでアクセスされた場合、本サーバーがHTTPS接続可能か確認する。HTTPS接続可能な場合はリダイレクトする
if (empty($_SERVER['HTTPS']) && !strstr($_SERVER['HTTP_HOST'], "127.0.0.1") && !strstr($_SERVER['HTTP_HOST'], "192.168.")) {
    $url = 'https://' . $_SERVER['HTTP_HOST'] . dirname($_SERVER['REQUEST_URI']) . '/script/index.html';
    $context = stream_context_create([
        "ssl" => [
            "verify_peer" => true,
            "verify_peer_name" => true,
        ],
    ]);
    $response = @file_get_contents($url, false, $context);
    if ($response !== false) {
        $url = 'https://' . $_SERVER['HTTP_HOST'] . dirname($_SERVER['REQUEST_URI']) . '/initialize.php';
        header("Location: $url");
        exit();
    }
}

if (file_exists($SYSTEM_SETTING_FILE) && file_exists($USER_SETTING_FILE)) {
    //設定ファイルが両方ある場合は初期化済み
    echo "初期化済みです";
    return;
} else if (file_exists($SYSTEM_SETTING_FILE) && !file_exists($USER_SETTING_FILE)) {
    //ユーザー / パスワード変更モード
    $message = '<span style="color:red">ユーザーID・パスワードを再設定します</span><br>';
    $next_step = "user_initialize";
} else {
    $message = '<img src="./resource/initlogo.png"><br>Simple Blog System PictPostPersonalへようこそ。<br>最初にユーザーIDとパスワードを設定します。<br>';
    $next_step = "all_initialize";
}

// stepなし：操作画面表示 stepあり:stepに応じた処理実行
$step = filter_input(INPUT_POST, 'step');

if ($step) {
    if ($step === "user_initialize" || $step === "all_initialize") {
        //フォルダ作成
        if (!file_exists("./env")) {
            mkdir("./env/", 0755);
            $fp = fopen("./env/index.html", "w");
            fclose($fp);
        }
        if (!file_exists("./images")) {
            mkdir("./images/", 0755);
            $fp = fopen("./images/index.html", "w");
            fclose($fp);
        }
        if (!file_exists("./images/upd")) {
            mkdir("./images/upd", 0755);
            $fp = fopen("./images/upd/index.html", "w");
            fclose($fp);
        }
        if (!file_exists("./env/thumbnails")) {
            mkdir("./env/thumbnails", 0755);
            $fp = fopen("./env/thumbnails/index.html", "w");
            fclose($fp);
            copyFolderContents("./resource/thumbnails", "./env/thumbnails");
        }

        //ユーザーID・パスワードの初期化
        $username = filter_input(INPUT_POST, 'username');
        $password = filter_input(INPUT_POST, 'password');
        $re_password = filter_input(INPUT_POST, 're_password');
        if ($password != $re_password || $username == "") {
            echo ("設定値が不正です");
            exit;
        }

        //パスワード設定
        $fp = fopen($USER_SETTING_FILE, "w");
        fwrite($fp, "<?php\n");
        fwrite($fp, '$sys_username="' .  preg_replace('/[\'"$]/', '\\\\$0', $username) . '";' . "\r\n");
        fwrite($fp, '$sys_password="' .   preg_replace('/[\'"$]/', '\\\\$0', password_hash($password, PASSWORD_DEFAULT)) . '";' . "\r\n");
        fclose($fp);

        //ログイン状態にする
        require_once("./token.php");
        publishToken("./", $username);
    }

    if ($step === "all_initialize") {
        //データベースの初期化
        $db_file_path = filter_input(INPUT_POST, 'dbfilepath');
        if (!$db_file_path) {
            // 指定がない場合はランダムなファイル名を生成
            $db_file_path = './env/' . bin2hex(random_bytes(rand(15, 20))) . '.db';
        }

        //DB構築
        $db = new SQLite3($db_file_path);
        $db->enableExceptions(true); //例外発生をONに設定
        $db->busyTimeout(5000); //ビジーハンドルを5秒に設定
        $retry = 0;
        while ($retry < 10) {
            try {
                $db->exec('PRAGMA journal_mode=wal');
                $retry = 99;
            } catch (Exception $ex) {
                sleep(1000);
                $retry++;
            }
        }
        if ($retry != 99) {
            echo "失敗";
            $db->close();
            exit;
        }

        $db->exec('begin Immediate');
        $db->exec("
        CREATE TABLE [accounts] (
            [id] INTEGER PRIMARY KEY AUTOINCREMENT,
            [url] TEXT,
            [icon] TEXT
            );

            INSERT INTO [accounts] ([id],[url],[icon]) VALUES ('1','','');
            INSERT INTO [accounts] ([id],[url],[icon]) VALUES ('2','','');
            INSERT INTO [accounts] ([id],[url],[icon]) VALUES ('3','','');
            INSERT INTO [accounts] ([id],[url],[icon]) VALUES ('4','','');
            INSERT INTO [accounts] ([id],[url],[icon]) VALUES ('5','','');
        ");

        $db->exec("
            CREATE TABLE [images] (
            [id]	INTEGER,
            [article_id]	INTEGER,
            [image_id] INTEGER,
            [sort]	INTEGER,
            [file_name]	TEXT,
            [width] INTEGER,
            [height] INTEGER,
            [thumb_l_name] TEXT,
            [thumb_l_width] INTEGER,
            [thumb_l_height] INTEGER,
            [thumb_name] TEXT,
            [thumb_width] INTEGER,
            [thumb_height] INTEGER,
            [type] TEXT DEFAULT 'image',
            [comment] TEXT DEFAULT '',
            PRIMARY KEY([id] AUTOINCREMENT)
        );
        create index articles_article_id on images(article_id);
        ");

        $db->exec("
            CREATE TABLE [articles] (
            [id] INTEGER PRIMARY KEY AUTOINCREMENT,
            [hidden] BOOLEAN,
            [created_at] DATETIME,
            [updated_at]  DATETIME,
            [title] TEXT,
            [comment] TEXT,
            [tags] TEXT,
            [color] TEXT,
            [use_html] BOOLEAN DEFAULT '0',
            [use_full_html] BOOLEAN DEFAULT '0',
            [style] INTEGERT DEFAULT '0',
            [pin] BOOLEAN DEFAULT '0',
            [nsfw] BOOLEAN DEFAULT '0',
            [like] INTEGER DEFAULT '0',
            [views] INTEGER DEFAULT '0',
            [from_ip] TEXT,
            [delete_key] TEXT,
            [use_alt_ogimage] BOOLEAN DEFAULT '0',
            [alt_url] TEXT,
            [show_password] TEXT DEFAULT '',
            [template] TEXT DEFAULT '',
            [thumbnail] TEXT,
            [sort] INTEGER '1'
            );
        ");

        $db->exec("
        CREATE TABLE [setting] (
            [id] INTEGER,
            [version] TEXT DEFAULT '0001',
            [site_title] TEXT,
            [display_name] TEXT,
            [mail_address] TEXT DEFAULT '',
            [thumbnail_type] INTEGER,
            [instruction_text] TEXT,
            [instruction_text_use_html]  DEFAULT '1',
            [twitter_card_text] TEXT,
            [nsfw]  DEFAULT '0',
            [show_title] BOOLEAN '0',
            [use_icon] BOOLEAN DEFAULT '0',
            [use_banner] BOOLEAN DEFAULT '0',
            [use_favicon] BOOLEAN DEFAULT '0',
            [use_ogimage] BOOLEAN DEFAULT '0',
            [image_color] TEXT,
            [use_static_generation]  DEFAULT '0',
            [static_index_file_path] TEXT,
            [static_log_file_dir] TEXT,
            [use_tag_cloud] BOOLEAN DEFAULT '1',
            [index_articles] INTEGER,
            [tweet_service] TEXT DEFAULT '',
            [tweet_service_key] TEXT DEFAULT '',
            [use_feedback] BOOLEAN DEFAULT '1',
            [show_likes] BOOLEAN DEFAULT '0',
            [move_link_type] INTEGER DEFAULT '1',
            [use_comment] BOOLEAN DEFAULT '0',
            [resize_border] INTEGER DEFAULT '1980',
            [exclude_index_list_tag] TEXT DEFAULT '',
            [use_guestwrite] BOOLEAN DEFAULT '0',
            [guest_attention] TEXT DEFAULT '',
            [use_portal_feed] BOOLEAN DEFAULT '0',
            [portal_feed_urls] TEXT DEFAULT '',
            [portal_tags] TEXT DEFAULT '',
            [portal_post_tags] TEXT DEFAULT '',
            [exclude_portal_post_tags] TEXT DEFAULT '',
            [use_mini_banner] BOOLEAN DEFAULT '0',
            [always_on_ssl] BOOLEAN DEFAULT '0',
            [use_rewrite] BOOLEAN DEFAULT '1', 
            [use_markdown] BOOLEAN DEFAULT '0',
            [use_mini_markdown] BOOLEAN DEFAULT '1',
            [index_all_images] BOOLEAN DEFAULT '0',
            [use_entry] BOOLEAN DEFAULT '0',
            [entry_password] TEXT DEFAULT '',
            [entry_text] TEXT DEFAULT '',
            [use_lightbox] BOOLEAN DEFAULT '0',
            [css_update] TEXT DEFAULT '',
            [default_style] TEXT DEFAULT '0',
            [contents_footer] TEXT DEFAULT '',
            [contents_template] TEXT DEFAULT '',
            [move_link_title] BOOLEAN DEFAULT '0',
            [tweet_use_clipboard] BOOLEAN DEFAULT '0',
            [write_exif] BOOLEAN DEFAULT '0',
            [title_template] TEXT DEFAULT '',
            PRIMARY KEY([id])
            );
            INSERT INTO [setting] (
                [id],
                [site_title],
                [display_name],
                [thumbnail_type],
                [instruction_text],
                [instruction_text_use_html],
                [twitter_card_text],
                [use_icon],
                [use_banner],
                [use_favicon],
                [image_color],
                [use_tag_cloud],
                [index_articles],
                [tweet_service])
            VALUES (
                '1',
                'タイトル',
                '@user_name',
                NULL,
                '',
                '1',
                '',
                '0',
                NULL,
                '0',
                '#51bb9b',
                '1', 
                28,
                'https://ppp.kannagi.net/tweet/');
        ");

        $db->exec("
        CREATE TABLE [activity] (
            [id] INTEGER PRIMARY KEY AUTOINCREMENT,
            [target] INTEGER,
            [created_at] DATETIME,
            [from_ip] TEXT,
            [action] TEXT,
            [read] BOOLEAN DEFAULT '0'
            );
        ");

        $db->exec("
        CREATE TABLE [templates] (
            [id] INTEGER PRIMARY KEY AUTOINCREMENT,
            [template_name] TEXT,
            [template_sort] INTEGER DEFAULT '10',
            [value] TEXT,
            [option1] TEXT,
            [option2] TEXT,
            [option3] TEXT,
            [option4] TEXT,
            [option5] TEXT,
            [option6] TEXT
            );
        ");

        $db->exec("
        CREATE TABLE [comments] (
            [id] INTEGER PRIMARY KEY AUTOINCREMENT,
            [target] INTEGER,
            [from_owner] BOOLEAN,
            [thread_id] INTEGER,
            [created_at] DATETIME,
            [from_ip] TEXT,
            [sender_name] TEXT,
            [sender_address] TEXT,
            [message] TEXT,
            [key_code] TEXT
            );
        ");
        $db->exec("insert into templates(template_name, template_sort, value, option1) values ('share_link', 100, 'twitter', 'https://twitter.com/intent/tweet?text=新しい記事を投稿しました%0a%0a%title% - %site_title%%0a%0a&url=%url%');");
        $db->exec("insert into templates(template_name, template_sort, value, option1) values ('share_link', 100, 'misskey.io', 'https://misskey.io/share?text=新しい記事を投稿しました%0a%0a%title% - %site_title%%0a%0a&url=%url%');");
        $db->exec("insert into templates(template_name, template_sort, value, option1) values ('share_link', 100, 'bsky.app', 'https://bsky.app/intent/compose?text=新しい記事を投稿しました%0a%0a%title% - %site_title%%0a%0a%url%');");
        $db->exec("insert into templates(template_name, template_sort, value) values ('exif', 100, '%Make% %Model% - %Lens% - f/%FNumber% - %SS% ISO%ISOSpeedRatings%');");
        $db->exec('commit');

        if (empty($_SERVER['HTTPS']) === false) {
            //HTTPS接続で初期化された場合はSSL-ON
            $db->exec("update setting set always_on_ssl = 1");
        }

        //設定ファイルにDBへのパスを記録
        $fp = fopen($SYSTEM_SETTING_FILE, "w");
        fwrite($fp, "<?php\n");
        fwrite($fp, '$system_db_file="' . $db_file_path . '";' . "\r\n");
        fclose($fp);

        //タグクラウドファイル生成
        $fp = fopen("./env/tagcloud.php", "w");
        fwrite($fp, "<?php\n");
        fwrite($fp, '$tagcloud_links = "";' . "\n");
        fwrite($fp, '$tagcloud_nolink = "";');
        fwrite($fp, '$tagcloud_links_more = "";' . "\n");
        fwrite($fp, '$tagcloud_nolink_more = "";');
        fwrite($fp, '$tagcloud_nolink_sys = "";');
        fwrite($fp, '$tagcloud_nolink_more_sys = "";' . "\n");
        fwrite($fp, '$tagcloud_list = "";');
        fclose($fp);

        //.htaccessファイル配置
        if (file_exists("./.htaccess")) {
            //既存のファイルがある場合はリネーム
            rename("./.htaccess", "./.htaccess.old." . bin2hex(random_bytes(rand(5, 8))));
        };
        copy("./resource/htaccess_base.txt", "./.htaccess");
        $file = fopen("./.htaccess", 'a');
        fwrite($file, "\r\n");
        fclose($file);
        sleep(1);

        //.htaccessが機能しているか確認する
        $protocol = isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https://' : 'http://';
        $url = $protocol . $_SERVER['HTTP_HOST'] . dirname($_SERVER['REQUEST_URI']) . '/rewrite_test_rewrite';
        $url = str_replace('//rewrite_test_rewrite', '/rewrite_test_rewrite', $url);
        $headers = get_headers($url);
        $httpCode = substr($headers[0], 9, 3);

        if ($httpCode == 404) {
            //rewriteが動作していない場合はフラグをOFFに
            $db->exec("update setting set use_rewrite = 0");
        } else {
            //rewriteが動作している場合はフラグをONに
            $db->exec("update setting set use_rewrite = 1");
        }

        //初期化プラグイン
        if (file_exists("./plugin/plugin_startup.php")) {
            require_once("./plugin/plugin_startup.php");
            $plugin_func = "plugin_startup\\initialize";
            $plugin_func($db);
        }
    }

    //USER CSSの初期化
    if (!file_exists("./env/user.css")) {
        copy("./resource/default_user.css", "./env/user.css");
    }



    //設定画面に移行
    header('Location: setting.php');
    exit;
}


function copyFolderContents($source, $destination)
{
    // ディレクトリを作成（存在しない場合）
    if (!is_dir($destination)) {
        mkdir($destination, 0755, true);
    }

    // ファイルおよびディレクトリの一覧を取得
    $items = scandir($source);
    $items = array_diff($items, array('.', '..'));

    // 各アイテムをコピー
    foreach ($items as $item) {
        $sourcePath = $source . '/' . $item;
        $destinationPath = $destination . '/' . $item;

        if (is_dir($sourcePath)) {
            // ディレクトリの場合、再帰的に中身をコピー
            copyFolderContents($sourcePath, $destinationPath);
        } else {
            // ファイルの場合、コピー
            copy($sourcePath, $destinationPath);
        }
    }
}

?>

<!doctype html>
<html lang="ja">

<head>
    <meta charset="utf-8">
    <title>初期設定</title>
    <meta name="description" content="">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <meta name="theme-color" content="#fafafa">
    <link rel="stylesheet" href="./style/recss.css">
    <link rel="stylesheet" href="./style/base.css">
    <link rel="stylesheet" href="./module/fontawesome/css/all.min.css" rel="preload">

    <script src="https://ajax.googleapis.com/ajax/libs/jquery/3.7.1/jquery.min.js"></script>

    <script src="./script/dialog.js"></script>
    <link rel="stylesheet" href="./style/dialog.css">

    <style>
        #container {
            display: flex;
            justify-content: center;
            align-items: center;

            width: 100%;
            height: 100vh;
        }

        #login-button {
            display: block;
            margin: 3em auto 0 auto;
        }

        table {
            margin: 0 auto 0 auto;
        }

        tr {
            height: 2.5em;
        }

        td {
            width: 15em;
        }

        #message {
            width: 100%;
            display: block;
            color: #f00;
            font-size: 0.9em;
            text-align: center;
            margin: 2em auto 2em auto;

        }

        label {
            margin-right: 3em;
        }

        .wmessage {
            color: red;
        }
    </style>
    <script>
        $(async function() {

            $('#password').on('input', function() {
                passwordCheck();
            });
            $('#re-password').on('input', function() {
                passwordCheck();
            });


            $('form').on('submit', async function(e) {
                if (!passwordCheck()) {
                    e.preventDefault();
                    await $("body").showDialog("ID/パスワードを確認してください");
                    return;
                }
                var wait = $('body').waitDialog("処理中…");
            });
        });

        function passwordCheck() {
            var err = false;
            var password = $("#password").val();
            var repassword = $("#re-password").val();
            var hasUpperCase = /[A-Z]/.test(password);
            var hasLowerCase = /[a-z]/.test(password);
            var hasNumbers = /\d/.test(password);
            if (password.length < 8) {
                $('#password-strength').html('パスワードは8文字以上である必要があります。');
                err = true;
            } else if (!hasUpperCase || !hasLowerCase || !hasNumbers) {
                $('#password-strength').html('パスワードには大文字・小文字・数字を含めてください。');
                err = true;
            } else if (password != repassword) {
                $('#password-strength').html("パスワードと再入力の内容が異なります");
                err = true;
            } else if ($("#username").val() == "") {
                $('#password-strength').html("ユーザーIDを入力してください");
            } else {
                $('#password-strength').html("");
            }

            if (err ||
                $("#username").val() == "" ||
                password == "" ||
                password != repassword) {
                return false;
            }
            return true;
        }
    </script>

</head>

<body>
    <div id="container">

        <div class="login-area">
            <?php echo $message ?>
            <br>
            <form action="./initialize.php" method="POST">
                <table>
                    <tr>
                        <td><label for="username">ユーザーID</label></td>
                        <td><input type="text" id="username" name="username" class="flat-input" value="<?= $DEMO_USER ?>"></td>
                    </tr>
                    <tr>
                        <td><label for="password">新しいパスワード</label></td>
                        <td><input type="password" id="password" name="password" class="flat-input" value="<?= $DEMO_PASSWORD ?>"></td>
                    </tr>
                    <tr>
                        <td><label for="password">新しいパスワード(再入力)</label></td>
                        <td><input type="password" id="re-password" name="re_password" class="flat-input" value="<?= $DEMO_PASSWORD ?>"></td>
                    </tr>
                </table>
                <span id="password-strength" class="wmessage"></span><br>
                ※パスワードは8文字以上・大文字・小文字・数字を含めてください。<br><br>
                <span style="font-size:0.9em; color:#449944;cursor:pointer" onClick="$('#options').toggle()">オプション設定▼</span>
                <div class="hide" id="options">
                    <table>
                        <tr>
                            <td>DBファイルパス</td>
                            <td><input type="text" id="dbfilepath" name="dbfilepath" class="flat-input" placeholder="通常は空で問題ありません"></td>
                        </tr>
                    </table>
                </div>
                <input type="hidden" name="step" value="<?php echo $next_step; ?>">
                <button name="submit" value="決定" class="button" id="login-button">決定</button>
            </form>
        </div>

</body>

</html>