//ダイアログボックス表示用JS
//別途 dialog.css を参照すること。
(function ($) {

  //1つまたは2つのボタンを持つダイアログを表示して待機する。
  //htmlStringで指定した内容をメッセージとして表示する。
  //button1を押すと"OK", button2を押すと"Cancel"を返却する。
  //button2が未定義または空文字の場合は1ボタンのみ表示される。
  //callbackが定義されると、メッセージボックス表示時にコールされる。
  $.fn.showDialog = function (htmlString, button1, button2, callback) {
    var body = $('body');
    var overlay = $('<div>').addClass('dialog-overlay');
    var dialog = $('<div>').addClass('dialog-box');

    var $hidden = $('<div style="display:none" class="dialog-tmp">').append(htmlString);
    body.append($hidden);
    var $contents_width = $hidden.width();
    $hidden.remove();


    var dialog_contents = $('<div>').addClass('dialog-contents');
    dialog_contents.append(htmlString);
    dialog.append(dialog_contents);

    if (($(window).width() / 2) < $contents_width) { dialog.css("min-width", "90vw"); }

    var $focusTarget;
    if (button2) {
      var $buttonGroup = $('<div>').addClass('dialog-buttons');
      var $okButton = $('<button>').addClass('dialog-button dialog-ok').text(button1);
      var $cancelButton = $('<button>').addClass('dialog-button dialog-cancel').text(button2);
      $buttonGroup.append($okButton);
      $buttonGroup.append($cancelButton);
      dialog.append($buttonGroup);
      $focusTarget = $cancelButton;
    } else {
      var $buttonGroup = $('<div>').addClass('dialog-buttons');
      var $okButton = $('<button>').addClass('dialog-button dialog-ok dialog-single-ok').text('OK');
      $buttonGroup.append($okButton);
      dialog.append($buttonGroup);
      $focusTarget = $okButton
    }

    overlay.append(dialog);
    body.append(overlay);
    $focusTarget.focus();

    return new Promise(function (resolve, reject) {
      if (callback) {
        callback();
      }

      $('.dialog-ok').click(function () {
        overlay.remove();
        resolve('OK');
      });
      if (button2) {
        $('.dialog-cancel').click(function () {
          overlay.remove();
          resolve('Cancel');
        });
      }
    });
  }

  //文章入力を待つダイアログを表示する。
  //title_msgで指定した内容をメッセージとして表示し、その下に入力欄が表示される。
  //button1を押すと入力内容, button2を押すと"Cancel"を返却する。
  //button2が未定義または空文字の場合は1ボタンのみ表示される。
  //callbackが定義されると、メッセージボックス表示時にコールされる。
  $.fn.inputDialog = function (title_msg, value, button1, button2, callback) {
    var body = $('body');
    var overlay = $('<div>').addClass('dialog-overlay');
    var dialog = $('<div>').addClass('dialog-box dialog-form');

    //サイズ調整
    var $hidden = $('<div style="display:none" class="dialog-tmp">').append(title_msg);
    body.append($hidden);
    var $contents_width = $hidden.width();
    $hidden.remove();

    var dialog_input = $('<textarea>').addClass('dialog-input');
    dialog_input.val(value);


    var dialog_contents = $('<div>').addClass('dialog-contents');
    dialog_contents.append(title_msg);
    dialog.append(dialog_contents);
    dialog.append(dialog_input);

    if (($(window).width() / 2) < $contents_width) { dialog.css("min-width", "90vw"); }


    if (button2) {
      var $buttonGroup = $('<div>').addClass('dialog-buttons');
      var $okButton = $('<button>').addClass('dialog-button dialog-ok input-ok').text(button1);
      var $cancelButton = $('<button>').addClass('dialog-button dialog-cancel').text(button2);
      $buttonGroup.append($okButton);
      $buttonGroup.append($cancelButton);
      dialog.append($buttonGroup);
    } else {
      var $buttonGroup = $('<div>').addClass('dialog-buttons');
      var $okButton = $('<button>').addClass('dialog-button dialog-ok dialog-single-ok').text('OK');
      $buttonGroup.append($okButton);
      dialog.append($buttonGroup);
    }

    overlay.append(dialog);
    body.append(overlay);
    $(".dialog-input").focus();
    return new Promise(function (resolve, reject) {
      if (callback) {
        callback();
      }

      $('.input-ok').click(function () {
        $ret = $('.dialog-input').val();
        overlay.remove();
        resolve($ret);
      });
      if (button2) {
        $('.dialog-cancel').click(function () {
          overlay.remove();
          resolve(false);
        });
      }
    });
  }

  //Tweet入力を待つダイアログを表示する。
  $.fn.tweetDialog = function (title_msg, value, button1, button2, callback) {
    var body = $('body');
    var overlay = $('<div>').addClass('dialog-overlay');
    var dialog = $('<div>').addClass('dialog-box dialog-form');

    //サイズ調整
    var $hidden = $('<div style="display:none" class="dialog-tmp">').append(title_msg);
    body.append($hidden);
    var $contents_width = $hidden.width();
    $hidden.remove();

    var dialog_input = $('<textarea>').addClass('dialog-input');
    dialog_input.val(value);
    var image_limit = $('<div class="dialog-check-area"><input type="checkbox" id="dialog-img-limit"><label for="dialog-img-limit">画像1枚目だけ投稿する</label></div>');


    var dialog_contents = $('<div>').addClass('dialog-contents');
    dialog_contents.append(title_msg);
    dialog.append(dialog_contents);
    dialog.append(dialog_input);
    dialog.append(image_limit);

    if (($(window).width() / 2) < $contents_width) { dialog.css("min-width", "90vw"); }


    if (button2) {
      var $buttonGroup = $('<div>').addClass('dialog-buttons');
      var $okButton = $('<button>').addClass('dialog-button dialog-ok input-ok').text(button1);
      var $cancelButton = $('<button>').addClass('dialog-button dialog-cancel').text(button2);
      $buttonGroup.append($okButton);
      $buttonGroup.append($cancelButton);
      dialog.append($buttonGroup);
      $cancelButton.focus();
    } else {
      var $buttonGroup = $('<div>').addClass('dialog-buttons');
      var $okButton = $('<button>').addClass('dialog-button dialog-ok dialog-single-ok').text('OK');
      $buttonGroup.append($okButton);
      dialog.append($buttonGroup);
      $okButton.focus();
    }

    overlay.append(dialog);
    body.append(overlay);

    return new Promise(function (resolve, reject) {
      if (callback) {
        callback();
      }

      $('.input-ok').click(function () {
        $ret = { "body": $('.dialog-input').val(), "limit": $('dialog-img-limit').prop("checked") };
        overlay.remove();
        resolve($ret);
      });
      if (button2) {
        $('.dialog-cancel').click(function () {
          overlay.remove();
          resolve('Cancel');
        });
      }
    });
  }
  //htmlStringで指定した内容をメッセージとして表示する。
  //制御はすぐに戻る。
  //呼出し元から <返却値>.close()が呼ばれると閉じる。
  //呼出し元から <返却値>.update(message)を呼ぶことで内容を書き換えられる。
  $.fn.waitDialog = function (htmlString) {
    var body = $('body');
    var overlay = $('<div>').addClass('dialog-overlay');
    var dialog = $('<div class="dialog">').addClass('dialog-box');


    var $hidden = $('<div style="display:none" class="dialog-tmp">').append(htmlString);
    body.append($hidden);
    var $contents_width = $hidden.width();
    $hidden.remove();


    var dialog_contents = $('<div>').addClass('dialog-contents');
    dialog_contents.append(htmlString);
    dialog.append(dialog_contents);

    if (($(window).width() / 2) < $contents_width) { dialog.css("min-width", "90vw"); }

    overlay.append(dialog);
    body.append(overlay);



    dialog.close = function () {
      overlay.remove();
      $(this).remove();
    }

    dialog.update = function (message) {
      dialog.html(message);
    }
    return dialog;
  }

  //imageUrlで指定した画像を画面いっぱいに表示して待機する。
  //画像がクリックされると閉じる。
  $.fn.imageDialog = function (imageUrl, width, height) {
    var body = $('body');
    var overlay = $('<div>').addClass('dialog-overlay').css('background-color', '#000');
    var dialog = $('<div>').addClass('dialog-image-box');
    var img = $('<img src="' + imageUrl + '" id="popupcontents" width="' + width + '" height="' + height + '">');
    dialog.append(img);

    if (width < ($(window).width()) &&
      height < ($(window).height())) {
      dialog.css({
        "display": "flex",
        "flex-direction": "column",
        "align-items": "center",
        "justify-content": "center"
      });
    }
    else if (height < ($(window).height())) {
      dialog.css({
        "display": "flex",
        "flex-direction": "column",
        "justify-content": "center"
      });
    } if (width < ($(window).width())) {
      dialog.css({
        "display": "flex",
        "flex-direction": "column",
        "align-items": "center",
      });
    }
    overlay.append(dialog);
    body.append(overlay);

    $('body').css({
      'overflow': 'hidden'
    });

    function onClose() {

      $('body').css({
        'overflow': 'auto'
      });
      overlay.remove();
    }

    dialog.focus();

    return new Promise(function (resolve, reject) {
      $('#popupcontents').click(function () {
        onClose();
        resolve('OK');
      });
      $('.dialog-image-box').click(function () {
        onClose();
        resolve('OK');
      });
    });
  }

  function dialog_noscroll(e) {
    e.preventDefault();
  }
})(jQuery);