
'use strict';
let guest_mode = '';
let max_size = 1980;

//イメージコメントの編集
function editImageComment(e) {
  let value = $(e.target).nextAll(".image-comment");
  $("body").inputDialog("コメント編集", value.val(), "OK", "キャンセル")
    .then(function (result) {
      if (result !== false) {
        value.val(result);
        unloadcheck_suppression = false;
      }
    });
}

let tapCount = 0;
function imageTap(e) {
  if (!tapCount) {
    ++tapCount;
    setTimeout(function () {
      tapCount = 0;
    }, 300);
  } else {
    e.preventDefault();
    tapCount = 0
    editImageComment(e);
  }
}

//ドラッグ&ドロップエリア イベント
function dragOver(e) {
  e.stopPropagation();
  e.preventDefault();
  $(this).css("border", "5px dotted #0B85A1");
}
function dragLeave(e) {
  e.stopPropagation();
  e.preventDefault();
  if ($(this).hasClass("dropArea")) {
    $(this).css("border", "5px dotted #ccc");
  }
  else {
    $(this).css("border", "");
  }
}
async function drop(e) {
  e.preventDefault();
  $("#dropArea").addClass("hide");
  $("#previewArea").removeClass("hide");
  let $dialog = $("base").waitDialog('<img src="./resource/loading.gif"><br>処理中…');
  if ($(this).hasClass("dropArea")) {
    $(this).css("border", "5px dotted #ccc");
    await createPreview(e.originalEvent.dataTransfer.files);
  }
  else {
    $(this).css("border", "");
    await createPreview(e.originalEvent.dataTransfer.files, $(this));
  }
  $dialog.close();
}

async function dropText(e) {
  e.preventDefault();
  $("#dropArea").addClass("hide");
  $("#previewArea").removeClass("hide");
  $(this).css("border", "");
  let $dialog = $("base").waitDialog('<img src="./resource/loading.gif"><br>処理中…');
  await createPreview(e.originalEvent.dataTransfer.files);
  $dialog.close();
}

//サムネイルの×ボタンから呼ばれる削除対象指定処理
function addRemoveImageList(target) {
  $("#removeimages").val($("#removeimages").val() + ' ' + target);
  setUnloadCheck();
}

//サムネイル右移動
function moveRight(target) {
  let pr = target.next(".preview-item");
  if (pr) {
    target.before(pr);
    setUnloadCheck();
  }
}

//サムネイル左移動
function moveLeft(target) {
  let pr = target.prev(".preview-item");
  if (pr) {
    target.after(pr);
    setUnloadCheck();
  }
}

//ファイルリード処理
function fileRead(file) {
  const promise = new Promise((resolve, reject) => {
    const reader = new FileReader();
    reader.onload = (event) => {
      resolve(event);
    }
    reader.readAsDataURL(file);
  });
  return promise;
}

//イメージリード処理
function imageRead(buffer) {
  const promise = new Promise((resolve, reject) => {
    const image = new Image();
    image.onload = (event) => {
      resolve(event)
    }
    image.targetFile = buffer.target;
    image.src = buffer.target.result;
  });
  return promise;
}

//EXIF取得
function exifRead(image) {
  const promise = new Promise((resolve, reject) => {
    EXIF.getData(image, function () {
      var exifData = EXIF.getAllTags(this);
      var result = {};

      //Exifライブラリのバグ対応
      if ("undefined" in exifData && !("Lens" in exifData)) {
        exifData["Lens"] = exifData["undefined"];
      }

      //露出時間から1/n表記取得
      if ("ExposureTime" in exifData) {
        if (exifData["ExposureTime"] > 1) {
          exifData["SS"] = exifData["ExposureTime"] + "sec";
        }
        else {
          exifData["SS"] = "1/" + (1 / exifData["ExposureTime"]);
        }
      }

      result.image = image;
      result.exif = exifData;
      resolve(result);
    });
  });
  return promise;
}

function formatExif(exifData) {
  return replaceVariables($exif_template, exifData);
}

// 正規表現を使用して%～%で囲まれた変数名を抽出
function replaceVariables(input, values) {
  const regex = /%\w+%/g;
  const matches = input.match(regex);
  if (matches) {
    matches.forEach(match => {
      const variableName = match.slice(1, -1);
      const replacement = values[variableName];
      if (replacement !== undefined) {
        input = input.replace(match, replacement);
      }
      else {
        input = input.replace(match, "---");
      }
    });
  }

  return input;
}

//ドラッグ or 選択されたファイルのプレビュー生成
async function createPreview(files, insertTarget = null) {
  setUnloadCheck();
  let inputs = [];
  let idx = 0;
  for (let i = 0; i < files.length; i++) {
    let file = files[i];
    if (file.type.startsWith('image/')) {
      //画像
      await createImagePreview(file, insertTarget);
    } else if (file.type.startsWith('video/')) {
      //動画
      await createMoviePreview(file, insertTarget);
    } else if (file.type.startsWith('text/')) {
      await uploadText(file);
    }
  }
}

async function uploadText(file) {
  var reader = new FileReader();
  let comment = '';
  reader.onload = function (event) {
    comment = $('#comment').val() + event.target.result;

    var match = comment.match(/^title:([^\n]+)/);
    if (match) {
      var titleContent = match[1].trim();
      $('#title').val(titleContent);
      comment = comment.replace(match[0], '').trim();
    }
    var match = comment.match(/^tag:([^\n]+)/);
    if (match) {
      var tagContent = match[1].trim();
      $('#tags').val(tagContent);
      comment = comment.replace(match[0], '').trim();
    }

    $('#comment').val(comment);
  };
  reader.readAsText(file);
}
async function createImagePreview(file, insertTarget = null) {

  const input = $('<input type="file" class="hide upload_images"  name="images[]">');
  const li = $('<li class="preview-item" data-filename="' + file.name + '"></li>');
  const img = $('<img class="preview-item-image" id="preview-' + file.name + '"  ondblClick="editImageComment(event)" ontouchend="imageTap(event)"; />');
  const comment = $('<input type="hidden" class="image-comment" name="image-comments[]" value="">')
  let deleteLink = $('<button href="#">×</button>');
  deleteLink.on("click", function (e) { e.preventDefault(); $(this).parent().remove(); });

  //プレビュー画像へのD&D対応
  li.on("dragover", dragOver);
  li.on("dragleave", dragLeave);
  li.on("drop", await drop);

  li.append(img, deleteLink, input, comment);


  if (('ontouchstart' in window) || (navigator.maxTouchPoints > 0) || (navigator.msMaxTouchPoints > 0)) {
    //タッチ操作用の左右並び替えボタン
    let rightLink = $('<i class="fa-regular fa-circle-rightmover"></i>').click(function () { moveRight($(this).parent()) });
    li.append(img, rightLink, input);
    let leftLink = $('<i class="fa-regular fa-circle-leftmovel"></i>').click(function () { moveLeft($(this).parent()) });
    li.append(img, leftLink, input);
  }

  if (insertTarget != null) {
    insertTarget.before(li);
  }
  else {
    $("#dropArea2Root").before(li);
  }

  //ファイルをReaderで読み込み、それをimageに展開後リサイズ処理判定
  await fileRead(file)
    .then(result => imageRead(result))
    .then(result => exifRead(result.target))
    .then(result => {
      if ("Model" in result.exif) {
        //EXIFが取得できている場合はコメントを設定する
        comment.val(formatExif(result.exif));
      }

      let readingImg = result.image;
      let width, height;

      if (max_size != 0 && (readingImg.width > max_size || readingImg.height > max_size)) {
        //画像の長辺がmax_sizePixを超えている場合はローカルリサイズする
        if (readingImg.width > readingImg.height) {
          let ratio = readingImg.height / readingImg.width;
          width = max_size;
          height = max_size * ratio;
        } else {
          let ratio = readingImg.width / readingImg.height;
          width = max_size * ratio;
          height = max_size;
        }
        let canvas = $("<canvas></canvas>")
          .attr("width", width)
          .attr("height", height);
        let ctx = canvas[0].getContext('2d', { preserveDrawingBuffer: true });
        ctx.clearRect(0, 0, width, height);
        ctx.drawImage(readingImg, 0, 0, readingImg.width, readingImg.height, 0, 0, width, height);

        let base64 = canvas.get(0).toDataURL("image/jpeg");
        let barr, bin, i, len;
        bin = atob(base64.split("base64,")[1]);
        len = bin.length;
        barr = new Uint8Array(len);
        i = 0;
        while (i < len) {
          barr[i] = bin.charCodeAt(i);
          i++;
        }
        let blob = new Blob([barr], { type: "image/jpeg" });
        let nfile = new File([blob], file.name)
        const dt = new DataTransfer();
        dt.items.add(nfile);
        input.prop("files", dt.files);
        input.data("filesize", nfile.size);
        console.log("client resize :" + file.name);

        img.attr({
          'src': base64
        });
      } else {
        //オリジナルのファイルを直接送信対象とするのでファイルサイズだけ記録
        input.data("filesize", file.size);
        const dt = new DataTransfer();
        dt.items.add(file);
        input.prop("files", dt.files);

        img.attr({
          'src': readingImg.src
        });
      }
    });
}

//動画ファイルリード処理
function videoFileRead(file) {
  const promise = new Promise((resolve, reject) => {
    const reader = new FileReader();
    reader.onload = (event) => {
      event.file = file;
      resolve(event);
    }
    reader.readAsArrayBuffer(file);
  });
  return promise;
}
//動画リード処理
function movieRead(buffer) {
  const promise = new Promise((resolve, reject) => {
    var thumbnailFrame = 1; // サムネイル取得秒指定
    var blob = new Blob([buffer.target.result], { type: buffer.file.type });
    var url = URL.createObjectURL(blob);
    var video = document.createElement('video');

    var snapImage = function () {
      var canvas = document.createElement('canvas');
      canvas.width = video.videoWidth;
      canvas.height = video.videoHeight;
      console.log(video.videoHeight);
      canvas.getContext('2d').drawImage(video, 0, 0, canvas.width, canvas.height);
      var image = canvas.toDataURL("image/png");
      var success = image.length > 100000;
      if (success) {
        const img = new Image();
        img.src = image;
        setTimeout(() => { // Safari対策
          URL.revokeObjectURL(url);
        }, 2000);
        return img;
      }
      else {
        currentCount += 1;
      }
      return null;
    }

    video.addEventListener("oncanplay", (event) => {
      if (event.duration > thumbnailFrame) {
        event.currentTime = thumbnailFrame;
      }
      else {
        event.currentTime = event.duration / 2;
      }
    });
    video.addEventListener('timeupdate', (event) => {
      var img = snapImage();
      if (img != null) {
        event.target.pause();
      }
      else if (currentCount == retryCount) {
        // サムネイル生成失敗
        console.log("make thumbnail failed");
        event.target.pause();
      }
      resolve(img);
    });
    video.preload = 'meta';
    video.src = url;
    video.muted = true;
    video.playsInline = true;
    video.currenttime = thumbnailFrame;
    video.play();
  });
  return promise;
}

async function createMoviePreview(file, insertTarget = null) {
  const input_preview = $('<input type="file" class="hide upload_images"  name="movie_previews[]">');
  const input_movie = $('<input type="file" class="hide upload_images"  name="movies[]">');
  const li = $('<li class="preview-item" data-filename="' + file.name + '"></li>');
  const img = $('<img id="preview-' + file.name + '"/>');
  let deleteLink = $('<button href="#">×</button>');
  deleteLink.on("click", function (e) { e.preventDefault(); $(this).parent().remove(); });
  li.append(img, deleteLink, input_preview, input_movie);
  //プレビュー画像へのD&D対応
  li.on("dragover", dragOver);
  li.on("dragleave", dragLeave);
  li.on("drop", await drop);

  if (('ontouchstart' in window) || (navigator.maxTouchPoints > 0) || (navigator.msMaxTouchPoints > 0)) {
    //タッチ操作用の左右並び替えボタン
    let rightLink = $('<i class="fa-regular fa-circle-rightmover"></i>').click(function () { moveRight($(this).parent()) });
    li.append(img, rightLink, input_preview);
    let leftLink = $('<i class="fa-regular fa-circle-leftmovel"></i>').click(function () { moveLeft($(this).parent()) });
    li.append(img, leftLink, input_preview);
  }
  if (insertTarget != null) {
    insertTarget.before(li);
  }
  else {
    $("#dropArea2Root").before(li);
  }

  //ファイルをReaderで読み込み、それをimageに展開後リサイズ処理判定
  await videoFileRead(file)
    .then(result => movieRead(result))
    .then(result => {

      //プレビュー画像を生成する
      let readingImg = result;
      let width, height;
      if (readingImg.width > readingImg.height) {
        let ratio = readingImg.height / readingImg.width;
        width = max_size;
        height = max_size * ratio;
      } else {
        let ratio = readingImg.width / readingImg.height;
        width = max_size * ratio;
        height = max_size;
      }
      if (width > readingImg.width) { width = readingImg.width; height = readingImg.height; }
      let canvas = $("<canvas></canvas>")
        .attr("width", width)
        .attr("height", height);
      let ctx = canvas[0].getContext('2d', { preserveDrawingBuffer: true });
      ctx.clearRect(0, 0, width, height);
      ctx.drawImage(readingImg, 0, 0, readingImg.width, readingImg.height, 0, 0, width, height);

      let base64 = canvas.get(0).toDataURL("image/jpeg");
      let barr, bin, i, len;
      bin = atob(base64.split("base64,")[1]);
      len = bin.length;
      barr = new Uint8Array(len);
      i = 0;
      while (i < len) {
        barr[i] = bin.charCodeAt(i);
        i++;
      }
      //プレビュー用ファイルセット
      let blob = new Blob([barr], { type: "image/jpeg" });
      let nfile = new File([blob], "image.jpg")
      const dt = new DataTransfer();
      dt.items.add(nfile);
      input_preview.prop("files", dt.files);
      input_preview.data("filesize", nfile.size);
      //動画本体セット
      const dt2 = new DataTransfer();
      dt2.items.add(file);
      input_movie.prop("files", dt2.files);
      img.attr({
        'src': base64
      });

    });
}

//非同期記事送信処理本体
async function postContents(form_data) {

  let url = "./api/article_post.php";
  if (guest_mode !== '') {
    url = "./api/guest_post.php";
  }

  return new Promise((resolve, reject) => {
    $.ajax({
      type: "POST",
      url: url,
      data: form_data,
      processData: false,
      contentType: false,
      success: function (response) {
        console.log('Success: ' + response);
        let result = JSON.parse(response);
        resolve(result)
      },
      error: function () {
        reject();
      }
    });
  });
}


//非同期画像アップロード処理
async function uploadFile(id, index, file, comment) {
  return new Promise((resolve, reject) => {

    let form_data = new FormData();
    form_data.append("images[]", file);
    form_data.append("comment", comment);
    form_data.append("id", id);
    form_data.append("index", index);
    $.ajax({
      url: "./api/image_post.php",
      type: "POST",
      data: form_data,
      dataType: "text",
      processData: false,
      contentType: false,

      success: function (response) {
        console.log("Success: " + response);
        let result = JSON.parse(response);

        resolve(result.file_name);
      },
      error: function (xhr, textStatus, errorThrown) {
        console.log("Error: " + errorThrown);
      }
    });
  });
};

//非同期削除処理本体
async function deleteLog(id) {
  let url = "./api/article_delete.php";
  let deletekey = "";
  if (guest_mode !== '') {
    url = "./api/guest_delete.php";
    deletekey = $('#delete_key').val();
  }
  return new Promise((resolve, reject) => {
    $.ajax({
      type: "POST",
      url: url,
      data: { "target": id, "delete_key": deletekey },
      success: function (response) {
        console.log("Success: " + response);
        var result = JSON.parse(response);
        resolve(result)
      },
      error: function () {
        reject();
      }
    });
  });
}
//非同期Tweet処理本体
async function postTweet(id, url) {

  let length = $(".preview-item").length;
  let addMsg = "";
  if (length > 4) {
    addMsg = "\r\n(他 " + (length - 4) + "枚の画像)\r\n"
  }
  var body = "新しい記事を投稿しました \r\n" + $("#title").val().replace(/<("[^"]*"|'[^']*'|[^'">])*>/g, '') + " - " + $(".sitetitle").html().replace(/<("[^"]*"|'[^']*'|[^'">])*>/g, '') + addMsg + " \r\n\r\n" + url;
  let result = await $("body").tweetDialog("Tweet", body, "OK", "キャンセル");
  if (result == "Cancel") {
    return;
  }
  return new Promise((resolve, reject) => {
    let wait = $('body').waitDialog('<img src="./resource/loading.gif"><p>Tweet中…</p>');
    $.ajax({
      type: "POST",
      url: "./api/tweet.php",
      data: {
        "id": id, "body": result['body'], "max_images": result['limit'] ? 1 : 4
      },
      success: async function (response) {
        console.log("Success: " + response);
        var result = JSON.parse(response);
        wait.close();
        if (result.status == "success") {
          await $("body").showDialog("ツイートしました！", "OK");
        }
        else {
          await $("body").showDialog(result.message, "OK");
        }

        resolve(result)
      },
      error: function () {
        wait.close();
        reject();
      }
    });
  });
}

let unloadcheck_suppression = false;
function setUnloadCheck() {
  if (!unloadcheck_suppression) {
    console.log("set");
    $(window).off('beforeunload');
    $(window).on('beforeunload', async function () {
      return "ページを移動しようとしています。\n入力した内容が失われますがよろしいですか？";
    });
  }
}
function unsetUnloadCheck() {
  console.log("unset");
  $(window).off('beforeunload');
}

async function openURLWithClipboardCopy(id, url) {
  var img = new Image();
  var apiUrl = "./api/common_api.php?command=getThumbnail&target=" + id;
  $.ajax({
    url: apiUrl,
    method: "GET",
    dataType: "json",
    success: function (response) {
      if (response.hasImage) {
        //画像ありの記事の場合はthmub_lサイズの画像をクリップボードにコピーしてURLを開く
        var imageUrl = './images/' + response.Row.thumb_l_name;
        img.onload = function () {
          const canvas = document.createElement('canvas');
          canvas.width = response.Row.thumb_l_width;
          canvas.height = response.Row.thumb_l_height;
          var ctx = canvas.getContext("2d");
          ctx.drawImage(img, 0, 0);
          canvas.toBlob(async (blob) => {
            const item = new ClipboardItem({
              'image/png': blob
            });
            await navigator.clipboard.write([item]);
            window.open(url, "_blank");
          });
        }
        img.src = imageUrl;
      } else {
        //画像がない記事の場合はそのままURLを開く
        window.open(url, "_blank");
      }
    },
    error: function (xhr, status, error) {
      window.open(url, "_blank");
    }
  });
}


$(async function () {

  $("form input, form textarea, form select").change(function () {
    setUnloadCheck();
  });


  $("#time_reload").on("click", function () {
    $('#created_at').val(getCurrentDateTime());
  });
  function getCurrentDateTime() {
    const now = new Date();

    const year = now.getFullYear();
    const month = String(now.getMonth() + 1).padStart(2, '0');
    const day = String(now.getDate()).padStart(2, '0');
    const hours = String(now.getHours()).padStart(2, '0');
    const minutes = String(now.getMinutes()).padStart(2, '0');
    const seconds = String(now.getSeconds()).padStart(2, '0');

    const formattedDateTime = `${year}-${month}-${day} ${hours}:${minutes}:${seconds}`;
    return formattedDateTime;
  }


  //削除ボタンクリック時
  $("#delete").on("click", async function (e) {
    e.preventDefault();
    let res = await $("body").showDialog("<p>この記事を削除してよろしいですか？\n(画像も削除されます）</p>", "OK", "キャンセル");
    console.log(res);
    if (res == "OK") {
      // 削除実行
      await deleteLog($(this).data("target"));
      if (guest_mode) {
        location.href = "./index.php";
      }
      else {
        location.href = "./console.php";
      }

    }
  });


  async function save(e, preview) {

    let guest_mode_flg = guest_mode !== '';
    e.preventDefault();

    //入力内容確認

    //削除キーの指定確認
    if (guest_mode_flg) {
      $("#delete_key").val($("#delete_key").val().trim());
    }
    if (guest_mode_flg && $("#delete_key").val() == "") {
      //削除キーが正しくない
      await $("body").showDialog("<p>削除キーを指定してください</p>", "OK");
      return;
    }
    //投稿日時の確認
    let datestring = $("#created_at").val();
    if (datestring != "") {
      let date1 = new Date(datestring.replace(/-/g, "/"));
      let date2 = new Date(datestring);
      if (isNaN(date1.getDate()) && isNaN(date2.getDate())) {
        //投稿日時が正しくない
        await $("body").showDialog("<p>投稿日時を正しく入力してください</p>", "OK");
        return;
      }
    }
    //代替URLの確認
    let alt_url = "";
    if ($('#alt_url').length > 0) {
      alt_url = $('#alt_url').val();
      if (alt_url.endsWith("/")) {
        alt_url = alt_url.slice(0, -1);
        $('#alt_url').val(alt_url);
      };
      if (alt_url.startsWith("/")) {
        alt_url = alt_url.slice(1);
        $('#alt_url').val(alt_url);
      };

      if (alt_url != "") {
        if (alt_url.indexOf("/") == 0) {
          //await $("body").showDialog("<p>/から始まる固定URLは使用できません</p>", "OK");
          //return;
        }
        if (alt_url.toLowerCase().indexOf("api/") >= 0) {
          await $("body").showDialog("<p>api/から始まる固定URLは使用できません</p>", "OK");
          return;
        }

        if (alt_url == "log") {
          await $("body").showDialog("<p>logは固定URLとして使用できません</p>", "OK");
          return;
        }

        let pattern = /^log\/\d+(\/.*)?$/;
        if (pattern.test(alt_url)) {
          await $("body").showDialog("<p>log/(数字)から始まる固定URLは使用できません</p>", "OK");
          return;
        }
      }
    }
    else {
      alt_url = "";
    }

    let newimage = false;

    let wait = $("body").waitDialog('<img src="./resource/loading.gif"><p>処理中…</p>');
    let msgtitle = '更新';
    if ($("#id").val() === undefined || $("#id").val() == "") {
      msgtitle = '投稿';
    }

    //ソート
    let sort = "";
    $(".preview-item").each(function (i, elem) {
      sort += $(elem).data("filename") + ";";
    });
    $("#sort").val(sort);

    //各値の設定      
    if ($("#title").val() == "") {
      $("#title").val("無題");
    }

    //送信元画像一覧
    let files = $(".upload_images");

    //合計画像ファイルサイズからアップロードモードを確定
    let totalFileSize = 0;
    let uploadMode = 0;
    if (files.length > 3 && !guest_mode_flg) {
      uploadMode = 1; //3枚以上は非同期送信(guestモードの時は同期送信のみ)
    }
    else {
      files.each(function () {
        let fileSize = parseInt($(this).data("filesize"));
        fileSize = 1;
        totalFileSize += fileSize;
      });
      if (totalFileSize > ($("#fileSelect").data("maxsize") * 0.9)) {
        uploadMode = 1;
      }
    }
    if (files.length > 0) {
      newimage = true;
    }

    //記事本体送信
    let form_data = new FormData($("#inputForm")[0]);
    //WAF対策のため本文をBASE64エンコード
    form_data.set('comment', btoa(unescape(encodeURIComponent($('#comment').val()))));


    if (uploadMode == 1) {
      //容量オーバーのため画像アップロードは別処理とするため
      //フォームデータからファイル類を削除
      form_data.delete('images[]');
    }

    let postResult = "";
    try {
      postResult = await postContents(form_data);
    }
    catch {
      wait.close();

      //投稿失敗
      let dialogText = '<p>' + msgtitle + 'に失敗しました</p>';
      $("body").showDialog(dialogText, 'OK');
      return;
    }
    console.log(JSON.stringify(postResult));
    if (postResult.status != "success") {
      wait.close();

      //投稿失敗
      let dialogText = '<p>' + msgtitle + 'に失敗しました</p>';
      $("body").showDialog(dialogText, 'OK');
      return;
    }
    let id = postResult.id;
    let index = postResult.image_index;
    if (uploadMode == 1) {
      //画像個別送信
      for (let i = 0; i < files.length; i++) {
        let file_data = files[i].files[0];
        let comment = $(files[i]).nextAll(".image-comment").val();
        if (file_data) {
          wait.update('<img src="./resource/loading.gif"><br>送信中… ' + i + ' / ' + files.length);
          console.log("update:" + id + "," + index);
          let newFile = await uploadFile(id, index, file_data, comment);
          $("#sort").val($("#sort").val().replace(file_data.name, newFile));
          index++;
        }
      }
      //並び替え情報更新
      let form_data = new FormData();
      form_data.append("sort", $("#sort").val());
      form_data.append("mode", "sort");
      form_data.append("id", id);
      let postResult = await postContents(form_data);
      if (postResult.status != "success") {
        //投稿失敗
        let dialogText = '<p>' + msgtitle + 'に失敗しました</p>';
        $("body").showDialog(dialogText, "OK");
      }

    }
    wait.close();

    unloadcheck_suppression = true;
    if (!preview) {
      unsetUnloadCheck();
      //Shereリンク生成
      let twitterintent = "";
      let dialogText = "";
      let length = $(".preview-item").length;
      var sharelink_html = "";
      for (var i = 0; i < $share_template.length; i++) {
        var url = $share_template[i].action;
        if ($copy_clipboard && !guest_mode_flg && length > 0) {
          let clickAction = " onClick='openURLWithClipboardCopy(" + id + ", \"" + url + "\");return false;'";
          sharelink_html += '<a href="#" target="_blank" ' + clickAction + '>' + $share_template[i].title + 'で共有</a><br>';
        }
        else {
          sharelink_html += '<a target="_blank" href="' + url + '">' + $share_template[i].title + 'で共有</a><br>';
        }
      }
      sharelink_html = sharelink_html.replace(/%title%/g, $("#title").val().replace(/<("[^"]*"|'[^']*'|[^'">])*>/g, ''));
      sharelink_html = sharelink_html.replace(/%site_title%/g, $(".sitetitle").html().replace(/<("[^"]*"|'[^']*'|[^'">])*>/g, ''));
      sharelink_html = sharelink_html.replace(/%url%/g, encodeURIComponent(postResult.url));

      if ($tweetmode == "api" && length >= 1 && !guest_mode_flg) {
        //Twitter連携設定があり、かつ画像あり
        dialogText = $('<div style="text-align: center;"></div>');
        let twitterintent_in = '<br><a style="font-size:0.7em" target="_blank" href="https://twitter.com/intent/tweet?text=新しい記事を投稿しました%0a%0a' +
          $("#title").val().replace(/<("[^"]*"|'[^']*'|[^'">])*>/g, '') + " - " + $(".sitetitle").html().replace(/<("[^"]*"|'[^']*'|[^'">])*>/g, '') + '%0a%0a&url=' +
          encodeURI(postResult.url) + '">[画像なしで共有]</a>';

        twitterintent = $('<span style="cursor:pointer">').addClass("tweetbutton").html('<i class="fa-brands fa-square-twitter" style="font-size:1em;color:#4898eb"></i>Twitterに共有</span>');
        //投稿完了メッセージ表示
        dialogText.append('<p style="margin-bottom:2em">' + msgtitle + 'しました！<br><a href="' + postResult.url + '" target="_blank">記事を確認</a></p>').append(twitterintent).append(twitterintent_in);
        $("body").showDialog(dialogText, "OK", null, async function () { $(".tweetbutton").on("click", async function () { await postTweet(id, postResult.url); }); })
          .then(function (result) {
            if (result === "OK" && (msgtitle === '投稿' || newimage)) {
              //新規投稿時は更新
              location.href = "./edit.php?id=" + postResult.id;
            }
          });
      }
      else {
        twitterintent = sharelink_html;
        dialogText = '<div style="text-align: center;"><p style="margin-bottom:2em">' + msgtitle + 'しました！<br><a href="' + postResult.url + '" target="ppp_check">記事を確認</a></p>' + twitterintent + '</div>';

        //投稿完了メッセージ表示
        $("body").showDialog(dialogText, "OK")
          .then(function (result) {
            if (result === "OK" && (msgtitle === '投稿' || newimage)) {
              if (guest_mode_flg) {
                location.href = "./guest_edit.php?id=" + postResult.id + "&delete_key=" + encodeURIComponent($("#delete_key").val());
              }
              else {
                location.href = "./edit.php?id=" + postResult.id;
              }
            }
            unloadcheck_suppression = false;
          });
      }
    }
    else {
      window.open(postResult.url, "ppp_check");
    }



  }

  //投稿・更新ボタンクリック時
  $("#submit").on('click', async function (e) { await save(e, false); });
  $("#submit2").on('click', async function (e) { await save(e, false); });
  $("#submit_preview").on('click', async function (e) { await save(e, true); });

  //クリップボードから画像ペースト処理
  $(document).on('paste', async function (e) {
    var items = e.originalEvent.clipboardData.items;
    for (var i = 0; i < items.length; i++) {
      if (items[i].type.indexOf('image') !== -1) {
        var blob = items[i].getAsFile();
        await createPreview([blob]);
        $("#dropArea").addClass("hide");
        $("#previewArea").removeClass("hide");
      }
    }
  });

  //ファイル選択時処理
  $("#fileSelect").on('change', async function (e) {
    $("#dropArea").addClass("hide");
    $("#previewArea").removeClass("hide");
    let $dialog = $("base").waitDialog('<img src="./resource/loading.gif"><br>処理中…');
    await createPreview($(this).prop("files"));
    $dialog.close();
    $(this).val(null); //このファイル選択欄は選択専用のためNULL値セット
  });

  //ドラッグ&ドロップエリア イベント
  $(".dropArea").on("dragover", dragOver);
  $(".dropArea").on("dragleave", dragLeave);
  $(".dropArea").on("drop", await drop);

  $(".dropArea").on("click", function (e) {
    $("#fileSelect").click();
  });
  $("#input-area").on("dragover", dragOver);
  $("#input-area").on("dragleave", dragLeave);
  $("#input-area").on("drop", await dropText);

  $(".tag").on("click", function (e) {
    let tag = $(this).html().replace("#", "");
    if (!$("#tags").val().includes(tag)) {
      $("#tags").val(($("#tags").val() + " " + tag).trim());
    }
  });
  $(".tag-cloud-more-link").on("click", function (e) {
    $(".tag-cloud-more").toggle(300);
  });

  $(window).on('popstate', function (e) {
    if (!e.originalEvent.state) return;
    loadContents(location.pathname);
  });

  //プレビュー画像を並び替え可能にする
  $("#previewList").sortable({ "axis": "x", "revert": "15", "tolerance": "pointer", });
  if (("ontouchstart" in window) || (navigator.maxTouchPoints > 0) || (navigator.msMaxTouchPoints > 0)) {
    $(".mover").css("display", "inline-block");
    $(".movel").css("display", "inline-block");
    $(".mover").click(function (e) { moveRight($(this).parent()) });
    $(".movel").click(function (e) { moveLeft($(this).parent()) });
  }
  //プレビュー画像へのD&D対応
  $(".preview-item").on("dragover", dragOver);
  $(".preview-item").on("dragleave", dragLeave);
  $(".preview-item").on("drop", await drop);

  $(document).on('keydown', 'form input', function (event) {
    if (event.keyCode == 13) {
      event.preventDefault();
      return false;
    }
  });
  $(document).keydown(function (e) {
    if (e.ctrlKey && (e.which == 83)) {
      e.preventDefault();
      $('#submit').click();
    }
  });
  //コマンドパレット

  $('#show_command_palette').on('click', function (e) {
    e.preventDefault();
    var palette_wait = $('body').waitDialog($('.command_palette').html());
    $(".insert_item").on('click', function () {
      var myField = document.getElementById('comment');
      var myValue = $(this).text();
      insertAtCursor(myField, myValue);
      palette_wait.close();
    });
    $(".command_palette_close").on('click', function () {
      palette_wait.close();
    });
  });
  function insertAtCursor(myField, myValue) {
    // IE support
    if (document.selection) {
      myField.focus();
      sel = document.selection.createRange();
      sel.text = myValue;
    }
    // MOZILLA/NETSCAPE support
    else if (myField.selectionStart || myField.selectionStart == '0') {
      var startPos = myField.selectionStart;
      var endPos = myField.selectionEnd;
      myField.value = myField.value.substring(0, startPos)
        + myValue
        + myField.value.substring(endPos, myField.value.length);
    } else {
      myField.value += myValue;
    }
  }
});

