<?php

/****************************************************************
 * 
 * 個別記事表示ページ
 * 　_GET
 *     *id  数値。表示する記事のID。必須。
 *     *staticmode 1だと静的出力モード。 log_{id}.htmlを吐き出す。
 * 
 *****************************************************************/
//共通処理呼び出し
$require_login = false; //ログイン不要
require_once("./common.php");


$target_id = ""; //この指定がある場合はこのIDの記事を扱う
$target_url = ""; //この指定がある場合はalt_urlがこのURLの記事を扱う

$url = explode('?', $PATH_INFO["system_request"])[0]; //引数より前のURLを取得
$pathParts = explode('/', $url);
$command = array_shift($pathParts);
$args = $pathParts;
$count = substr_count($url, "/"); // "/"の数を数える
if ($count > 0) {
  $rewrite_rel  = str_repeat("../", $count); // "../"を数だけ繰り返す
} else {
  $rewrite_rel = "./";
}
$static_mode = 0;
$arg_count = count($args);
if ($command == "rewrite_test_rewrite") {
  //rewriteが使用できるかどうかの確認用コマンド
  //この場合は"rewrite"のみを出力して終了する
  echo "rewrite";
  exit();
} else if ($command == "log" && $arg_count > 0 && is_numeric($args[0])) {
  // ～/log/1111 の場合はその番号の記事を表示
  $target_id =  $args[0];

  //2番目の引数はページインデックス番号(～/log/1111/3)
  if (count($args) >= 2) {
    $index = $args[1];
  } else {
    $index = 1;
  }
  $static_mode = 0;
} else if ($command != "view.php") {
  // view.php以外のURLが渡された場合はalt_urlとして処理する
  if ($command == "log") {
    if ($arg_count == 0 || $args[0] == "index.php") {
      // ～/log/ の場合および ～/log/index.php の場合はトップページに移動
      header("Location: ../index.php");
      exit;
    }
  }
  $target_url = $url;
  $index =  filter_input(INPUT_GET, 'page', FILTER_VALIDATE_INT) ?? 1;
} else {
  // view.phpへのアクセスの場合は従来通りGET引数から判断する
  $rewrite_rel = "";

  //引数読み込み
  $target_id = filter_input(INPUT_GET, 'id', FILTER_SANITIZE_NUMBER_INT); //処理対象Logs-ID
  $static_mode = filter_input(INPUT_GET, 'static_mode', FILTER_SANITIZE_NUMBER_INT) ?? 0; //静的出力モード
  $index =  filter_input(INPUT_GET, 'page', FILTER_VALIDATE_INT) ?? 1; //ページインデックス
}

//パスワード保護記事用 閲覧パスワード
$input_password = filter_input(INPUT_POST, 'password') ?? "";

if ($target_url == "" && $target_id == "") {
  //ID指定またはalt_url指定がない場合は404とする
  header("HTTP/1.0 404 Not Found");
  if (file_exists("404.php")) {
    ob_start();
    include("404.php");
    $result  = ob_get_contents();
    ob_end_clean();
    $result = str_replace('./', $rewrite_rel, $result);
    echo $result;
    exit;
  } else {
    echo "404 Page Not Found ( $url )";
  }
  exit;
}

//logsテーブルから記事を検索
try {
  if ($target_id != "") {
    $stmt = $db->prepare('SELECT * From articles Where id= :id');
    $stmt->bindValue(':id',  $target_id);
  } else {
    $stmt = $db->prepare('SELECT * FROM articles where alt_url = :alt_url');
    $stmt->bindValue(":alt_url",  $target_url);
  }
  $result = $stmt->execute();

  if (!$row = $result->fetchArray()) {
    //指定されたログが存在しない
    $db->close();
    header("HTTP/1.0 404 Not Found");
    if (file_exists("404.php")) {
      ob_start();
      include("404.php");
      $result  = ob_get_contents();
      ob_end_clean();
      $result = str_replace('./', $rewrite_rel, $result);
      echo $result;
      exit;
    } else {
      echo "404 Page Not Found ( $url )";
    }
    exit();
  }
} catch (Exception $ex) {
  $db->close();
  echo 'システムエラーが発生しました<br>';
  $error = $ex->getTraceAsString();
  exit;
}

//NSFW確認
$nsfw = $row['nsfw'] ?? false;
if ($nsfw && $NSFW_CONSENT == "0") {
  //NSFWコンテンツだが同意がとれていない

  //クローラー判定→クローラー相手の場合はNSFW警告しない
  $userAgent = $_SERVER['HTTP_USER_AGENT'];
  $isCrawler = preg_match('/bot|crawl|slurp|spider|twitter|misskey|bsky/i', $userAgent);
  if (!$isCrawler) {
    header("Location: {$rewrite_rel}nsfw.php?request_id=" . $row['id']);
    exit;
  }
}

//Entry確認
$show_password = $row['show_password'] ?? '';
if ($show_password != '') {
  $entry = filter_input(INPUT_COOKIE, "entry_consent[$target_id . $target_url]") ?? "";
  if (!password_verify($row['show_password'], $entry)) {
    //エントリーが必要なコンテンツだが同意がとれていない
    header("Location: {$rewrite_rel}entry.php?title=" . h($row['title']) . "&request=" . $target_id . $target_url);
    exit;
  }
}

//表示パスワード確認
$password = $row['show_password'] ?? '';
$password_mode = false;
if ($password != '' && $input_password != $password) {
  //パスワードが必要だがパスワードが渡されていない
  $password_mode = true;
}


//targetをidに書き換え
$target = $row['id'];

//テンプレートに引き渡す情報の構築
$use_alt_ogimage = $row["use_alt_ogimage"] ?? false;
$article = array();
$article['id'] = $target;
$article['nsfw'] = false;
$article['title'] = $row['title'];
$article['created_at'] =  h($row['created_at']);
$article['updated_at'] = h($row['updated_at'] ?? "");
$article['views'] = $row['views'];
$article['like'] = $row['like'];

if (strstr($article['created_at'], "9999")) {
  //更新日付が「9999」の場合は非表示にする
  $article['created_at'] = "";
}

$use_user_thumbnail = ($row['thumbnail']  == "@user_thumbnail"); //1枚目をサムネイルとして扱うか
$style = $row['style']; //記事のスタイル

if ($style == 0 || $style == 4 || $style == 5) {
  //シンプル or 漫画
  $simple_mode = true;
} else {
  $simple_mode = false;
}

//ゲスト記事対応
if ($row['delete_key'] && $row['delete_key'] != "") {
  $delete_link = '<form id="guest_delete_form" action="./api/guest_delete.php" method="GET">削除キー：<input type="hidden" name="target" value="' . $target . '"><input class="flat-input" style="width:5em" type="password" name="delete_key"><button style="background-color:#CCC;margin-left:1em;padding:0.5em;font-size:0.8em;border-radius:0.5em">削除</button></form>';
} else {
  $delete_link = "";
}

//イイネ数
if ($SETTING['show_likes']) {
  if ($row['like'] ?? 0 == 1) {
    $article['show_likes'] = "<span class='show_like_count'><i id='like-heart-show' class='fa-heart fa-solid'></i> {$row['like']}</span><br>";
  } else {
    $article['show_likes'] = "";
  }
} else {
  $article['show_likes'] = "";
}

//タグクラウド
$primary_tag = "";
$tags = "";
$tags_raw = array();
foreach (explode(' ', $row['tags']) as $tag) {
  if ($tag != "") {
    if (substr($tag, 0, 1) == "-") {
      continue;
      //$tag = substr($tag, 1);
    }
    $tags = $tags . "<a href='./list.php?q=%23{$tag}'>#{$tag}</a> ";
    $tags_raw[] = $tag;
    if ($primary_tag == "") {
      //プライマリタグ
      $primary_tag = $tag;
    }
  }
}
$article["tags"] = $tags;


//前後移動

//前後リンク
$prev = "";
$next = "";
if ($SETTING["move_link_type"] > 0) {
  $base_created_at = $article['created_at']; //この記事の作成日時をベースに前後の記事を検索する
  try {
    //次の行を取得する
    switch ($SETTING["move_link_type"]) {
      case 1:
        //無条件
        $nextdb = $db->prepare('SELECT * From articles Where hidden != 1 and style != 99 and style != 98 and  created_at > :created_at order by created_at asc limit 1');
        $nextdb->bindValue(':created_at',  $base_created_at);
        $prevdb = $db->prepare('SELECT * From articles Where hidden != 1 and style != 99 and style != 98 and  created_at < :created_at order by created_at desc limit 1');
        $prevdb->bindValue(':created_at',  $base_created_at);
        break;

      case 3:
        //プライマリタグで前後
        $nextdb = $db->prepare('SELECT * From articles Where hidden != 1 and style != 99 and style != 98 and  created_at > :created_at and tags like :tag order by created_at asc limit 1');
        $nextdb->bindValue(':created_at',  $base_created_at);
        $nextdb->bindValue(':tag',  "%" . $primary_tag . "%");
        $prevdb = $db->prepare('SELECT * From articles Where hidden != 1 and style != 99 and style != 98 and  created_at < :created_at and tags like :tag order by created_at desc limit 1');
        $prevdb->bindValue(':created_at',  $base_created_at);
        $prevdb->bindValue(':tag',  "%" . $primary_tag . "%");
        break;
      case 2:
        //タグの完全一致で前後
        $where = "";
        foreach ($tags_raw as $idx => $tag) {
          $where = $where . " and tags like :tag$idx";
        }

        $nextdb = $db->prepare('SELECT * From articles Where hidden != 1 and style != 99 and style != 98 and  created_at > :created_at ' . $where . ' order by created_at asc limit 1');
        $nextdb->bindValue(':created_at',  $base_created_at);
        $prevdb = $db->prepare('SELECT * From articles Where hidden != 1 and style != 99 and style != 98 and  created_at < :created_at ' . $where . ' order by created_at desc limit 1');
        $prevdb->bindValue(':created_at',  $base_created_at);
        foreach ($tags_raw as $idx => $tag) {
          $nextdb->bindValue(":tag$idx",  "%" . $tag . "%");
          $prevdb->bindValue(":tag$idx", "%" . $tag . "%");
        }
        break;
    }
    //次の行を取得する
    $result = $nextdb->execute();

    if (!$sub_row = $result->fetchArray()) {
      //新しい行が存在しない
      $next = "-";
    } else {
      $move_caption = "次へ";
      if ($SETTING["move_link_title"] ?? '' == '1') {
        $move_caption = h($sub_row["title"]);
      }
      if ($SETTING["use_rewrite"]) {
        if ($sub_row["alt_url"]) {
          $next = '<a href="./' . h($sub_row['alt_url']) . '?back_url=' . $BACK_URL . '">' .  $move_caption . '</a>';
        } else {
          $next = '<a href="./log/' . h($sub_row['id']) . '?back_url=' . $BACK_URL . '">' .  $move_caption . '</a>';
        }
      } else {
        $next = '<a href="./view.php?id=' . h($sub_row['id']) . '&back_url=' . $BACK_URL . '">' .  $move_caption . '</a>';
      }
    }
    //前の行を取得する
    $result = $prevdb->execute();
    if (!$sub_row = $result->fetchArray()) {
      //新しい行が存在しない
      $prev = "-";
    } else {
      $move_caption = "前へ";
      if ($SETTING["move_link_title"] ?? '' == '1') {
        $move_caption = h($sub_row["title"]);
      }
      if ($SETTING["use_rewrite"]) {
        if ($sub_row["alt_url"]) {
          $prev = '<a href="./' . h($sub_row['alt_url']) . '?back_url=' . $BACK_URL . '">' .  $move_caption . '</a>';
        } else {
          $prev = '<a href="./log/' . h($sub_row['id']) . '?back_url=' . $BACK_URL . '">' .  $move_caption . '</a>';
        }
      } else {
        $prev = '<a href="./view.php?id=' . h($sub_row['id']) . '&back_url=' . $BACK_URL . '">' .  $move_caption . '</a>';
      }
    }
  } catch (Exception $ex) {
    $db->close();
    echo 'システムエラーが発生しました<br>';
    $error = $ex->getTraceAsString();
    exit;
  }
}

//コメント
$comment = $row['comment'] ?? "";
$comment = $comment . $SETTING['contents_footer'] ?? "";
$comment = str_replace('{{', "&#060;&#060;", $comment); //{{ 表記をエスケープ
$comment = str_replace('}}', "&#062;&#062;", $comment); //}} 表記をエスケープ
$comment = str_replace('<<', "&lt;", $comment); //{{ 表記をエスケープ
$comment = str_replace('>>', "&gt;", $comment); //}} 表記をエスケープ
if ($style == 4) {
  //漫画
  $comment = $comment . "{mangaview}";
} else if ($style == 5) {
  //漫画
  $comment = $comment . "{mangaview r}";
}

preg_match('/{template (.*?)}/i', $comment, $match);
if ($match) {
  $template_name = $match[1];
  $comment = preg_replace('/{template (.*?)}\r?\n?/i', '', $comment);
} else {
  $template_name = $row['template'] ?? "";
}


//パスワードによる非表示置換を実施
if ($password_mode) {
  //未認証→{no-password}を表示
  $comment = preg_replace('/{no-password}(.*){\/no-password}/is', '$1', $comment);
} else {
  //認証済み→{no_password}を非表示に
  $comment = preg_replace('/{no-password}(.*){\/no-password}/is', '', $comment);
}

//リンクを先行変換
$comment = preg_replace('/{link:http(.*?) (.*?)}/i', '<a href="http$1" target="_blank">$2</a>', $comment);
$comment = preg_replace('/{link:http(.*?)}/i', '<a href="http$1" target="_blank">http$1</a>', $comment);
$comment = preg_replace('/{link http(.*?) (.*?)}/i', '<a href="http$1" target="_blank">$2</a>', $comment);
$comment = preg_replace('/{link http(.*?)}/i', '<a href="http$1" target="_blank">http$1</a>', $comment);
if ($SETTING["use_rewrite"]) {
  $comment = preg_replace('/{link:([0-9]*?) (.*?)}/i', '<a href="./log/$1">$2</a>', $comment);
  $comment = preg_replace('/{link:(.*?) (.*?)}/i', '<a href="./log/$1">$2</a>', $comment);
  $comment = preg_replace('/{link ([0-9]*?) (.*?)}/i', '<a href="./log/$1">$2</a>', $comment);
  $comment = preg_replace('/{link (.*?) (.*?)}/i', '<a href="./log/$1">$2</a>', $comment);
} else {
  $comment = preg_replace('/{link:([0-9]*?) (.*?)}/i', '<a href="./view.php?id=$1">$2</a>', $comment);
  $comment = preg_replace('/{link:(.*?) (.*?)}/i', '<a href="./view.php?id=$1">$2</a>', $comment);
  $comment = preg_replace('/{link ([0-9]*?) (.*?)}/i', '<a href="./view.php?id=$1">$2</a>', $comment);
  $comment = preg_replace('/{link (.*?) (.*?)}/i', '<a href="./view.php?id=$1">$2</a>', $comment);
}

//Markdownを処理
if ($SETTING["use_markdown"]) {
  require_once("./module/Parsedown.php");
  require_once("./module/ParsedownEx.php");

  $Parsedown = new ParsedownEx();
  $comment = $Parsedown->text($comment);
} else if ($SETTING["use_mini_markdown"]) {
  // 一部マークダウンだけ対応する
  // 見出し
  $pattern = '/^(#+)\s*(.*)/m';
  $comment = preg_replace_callback($pattern, function ($matches) {
    $level = strlen($matches[1]);
    return "<h{$level}  class=\"md\">{$matches[2]}</h{$level}>";
  }, $comment);

  // 強調
  $pattern = '/\*\*(.*?)\*\*/';
  $replace = '<strong class="md">\\1</strong>';
  $comment = preg_replace($pattern, $replace, $comment);

  // 斜体
  $pattern = '/\*(.*?)\*/';
  $replace = '<em class="md">\\1</em>';
  $comment = preg_replace($pattern, $replace, $comment);

  //テーブル
  $comment = markdownTableToHtml($comment);
}

if (!$row['use_html']) {
  //通常モード = タグをエスケープする
  $comment = h($comment);
}

//キャプションを取り出し
$caption = "";
$comment = preg_replace('/{\/caption}\r\n/i', '{/caption}', $comment);
$comment = preg_replace('/{\/caption}\n/i', '{/caption}', $comment);
preg_match('/{caption}(.*?){\/caption}/si', $comment, $match);
if ($match) {
  $caption = $match[1];
  $caption = convertLineBreaks($caption);
  $comment = preg_replace('/{caption}(.*?){\/caption}\r?\n?/si', '', $comment);
}

//画像一覧を取得
try {
  $stmt = $db->prepare('SELECT * FROM images Where article_id= :id Order by sort ASC');
  $stmt->bindValue(':id',  $target);
  $result = $stmt->execute();
  $previews = "";
  $firstPreview = "";

  if ($use_user_thumbnail) {
    //1枚目をサムネイル扱いとして読み捨てる
    $row = $result->fetchArray();
    if ($row) {
      //1枚目の画像をTwitterCard サムネイル用とする
      $firstPreview = h($row["thumb_l_name"]);
    }
  }

  $idx = 1;
  $images = array();
  if ($simple_mode) {
    //ノーマルモード 先頭からソート順に画像を表示
    while ($row = $result->fetchArray()) {
      $img_comment = $row['comment'] ?? "";
      if ($img_comment != "") {
        $img_comment = convertLineBreaks($img_comment);
        $img_comment = "<span class='image-comment'>" . $img_comment . "</span>";
      }
      $lightbox_link_a = '';
      $lightbox_link_b = '';
      if ($SETTING['use_lightbox']) {
        $lightbox_link_a = '<a class="lighbox-link" data-lightbox="images" href="./images/' . $row["file_name"] . '">';
        $lightbox_link_b = '</a>';
      }
      $thumb_file_name = h($row["thumb_l_name"]);
      if ($firstPreview == "") {
        //1枚目の画像をTwitterCard サムネイル用とする
        $firstPreview =  $thumb_file_name;
      }
      if ($row['type'] == "video") {
        $img = '<div class="view-image-container"><video controls class="view-image" src="./images/' . h($row["file_name"])  . '"' .
          ' data-src="./images/' .  h($row["file_name"])  . '" width=' .  h($row["thumb_l_width"]) . ' height=' .  h($row["thumb_l_height"]) .
          ' data-width="' . h($row["width"]) . '" data-height="' . h($row["height"]) . '">' . $img_comment . '</div>';
      } else {
        $img = '<a name="' . $idx . '" id="' . $idx . '"></a><div class="view-image-container">' . $lightbox_link_a . '<img class="view-image manga-view-page" src="./images/' . $thumb_file_name  . '"' .
          ' data-src="./images/' .  h($row["file_name"])  . '" width=' .  h($row["thumb_l_width"]) . ' height=' .  h($row["thumb_l_height"]) .
          ' data-width="' . h($row["width"]) . '" data-height="' . h($row["height"]) . '">' . $lightbox_link_b .  $img_comment . '</div>';
      }
      $previews = $previews . $img;
      $idx++;
    }
  } else {
    //HTML内で画像を指定するため、画像一覧を作成する
    while ($row = $result->fetchArray()) {
      if ($firstPreview == "") {
        //1枚目の画像をTwitterCard サムネイル用とする
        $firstPreview = h($row["thumb_l_name"]);
      }
      $images[$idx] = $row;
      $idx++;
    }
  }
  $article["previews"] = $previews;
} catch (Exception $ex) {
  $db->close();
  echo 'システムエラーが発生しました<br>';
  $error = $ex->getTraceAsString();
  exit;
}

//小説用
if ($style == 2) {
  $comment = "{ht}" . $comment . "{/ht}";
} else if ($style == 3) {
  $comment = "{vt}" . $comment . "{/vt}";
}
//改行を<BR>に変換
$comment = convertLineBreaks($comment);

//特殊記述を処理する
$comment = convertCommands($comment, $images, $index, $next, $prev);

//Lightboxを使用する場合、そのスクリプトを読み込み、imageDialog処理を無効化する
if ($SETTING['use_lightbox']) {
  $comment = $comment . '<link rel="stylesheet" href="./module/lightbox/css/lightbox.min.css"><script src="./module/lightbox/js/lightbox.min.js"></script><script> $.fn.imageDialog = function (imageUrl, width, height) {}; </script>';
}

$comment = str_replace("&#060;&#060;", '{',  $comment); //{{ 表記のエスケープを復元
$comment = str_replace("&#062;&#062;", '}',  $comment); //}} 表記のエスケープを復元
$article['comment'] = $comment;


//固定ページ用メニュー生成
$home_menu = "";
$accounts = array();
//メニュー読み取り
$menus = [];
$home_menu = "";
$result = $db->query('SELECT id, created_at, title, comment, tags, nsfw, alt_url From articles Where style = 99 ORDER BY created_at DESC');
while ($row = $result->fetchArray()) {
  if ($SETTING["use_rewrite"]) {
    if ($row['alt_url']) {
      $url = "./{$row['alt_url']}";
    } else {
      $url = "./log/{$row['id']}";
    }
  } else {
    $url = "./view.php?id={$row['id']}";
  }
  if ($row['id'] == $target) {
    $home_menu = $home_menu . "<li  class='home_menu_item home_menu_item_current' data-id='{$row['id']}' data-url='{$row['alt_url']}'>{$row['title']}</li>";
    $menus[] =  ['id' => $row['id'], 'url' =>  $url, 'title' => $row['title'], 'is_current' =>  1];
  } else {
    $home_menu = $home_menu . "<a href='$url' class='home_menu_item_a'><li  class='home_menu_item' data-id='{$row['id']}' data-url='$url'>{$row['title']}</li></a>";
    $menus[] =  ['id' => $row['id'], 'url' =>  $url, 'title' => $row['title'], 'is_current' =>  0];
  }
}

if ($home_menu != "") {
  array_unshift($menus, ['id' => '-1', 'url' =>  './index.html', 'title' => 'title', 'is_current' =>  0]);
  $home_menu = "<ul class='home_menu'><a href='./index.php' class='home_menu_item_a'><li class='home_menu_item' data-id='home'>HOME</li></a>$home_menu</ul>";
}

//SNSリンク
$account_rows = $db->query('SELECT * FROM accounts');
while ($row = $account_rows->fetchArray()) {
  $accounts[] = $row;
}



//表示モードの確定
// HTML FULLの場合 : タイトル / タグ / コメント（HTML）
// 画像がある場合　 : タイトル / コメント / タグ / 画像
// 画像がない場合　 : タイトル / タグ / コメント
if ($simple_mode && $previews != "") {
  $mode = "normal";
} else if ($simple_mode && $previews == "") {
  $mode = "text";
} else {
  $mode = "advance";
}
$article['mode'] = $mode;

//Twitterカード用データの構築
$document['card_title'] = $article['title'] . " - " . $SETTING['site_title'];
if ($caption == "") {
  $card_comment = preg_replace("/<script>.*<\/script>/si", "", $comment);
  $card_comment = preg_replace("/<style>.*<\/style>/si", "", $card_comment);
  $card_comment = preg_replace("/\r|\n|\r\n/", "", strip_tags($card_comment));
  if (mb_strlen($card_comment) > 100) {
    $card_comment = mb_substr($card_comment, 0, 100, 'UTF-8') . '…';
  }
} else {
  $card_comment = preg_replace("/\r|\n|\r\n/", "", strip_tags($caption));
}
$document['card_comment'] = $card_comment;
$document['card_type'] = "summary";
$document['card_thumbnail'] = "";
if ($nsfw || $use_alt_ogimage) {
  //NSFWコンテンツの場合　または画像表示しないオプションの場合はogimage
  $article['nsfw'] = true;
  if ($SETTING['use_ogimage']) {
    $document['card_type'] = "summary_large_image";
    $document['card_thumbnail'] = $PATH_INFO["url_system_root"] . '/images/ogimage.jpg';
  }
} else {
  //画像ありの場合は1枚目を指定
  if ($firstPreview != "") {
    $document['card_type'] = "summary_large_image";
    $document['card_thumbnail'] = $PATH_INFO["url_system_root"] . '/images/' . $firstPreview;
  } else if ($SETTING['use_ogimage']) {
    $document['card_type'] = "summary_large_image";
    $document['card_thumbnail'] = $PATH_INFO["url_system_root"] . '/images/ogimage.jpg';
  } else if ($SETTING['use_banner']) {
    $document['card_type'] = "summary_large_image";
    $document['card_thumbnail'] = $PATH_INFO["url_system_root"] . '/images/banner.jpg';
  }
}
$document['card_thumbail'] = $document['card_thumbnail']; //バグ対処（テンプレート互換性維持のため）


//UIレンダリング
require_once('./module/NoSixTemplate.php');
if ($template_name != "" && file_exists("./env/templates/template_view_$template_name.php")) {
  //テンプレート指定がある 
  $template = new NoSixTemplate("template_view_$template_name.php", './env/templates');
} else if ($style == 99 || $style == 98) {
  if (file_exists('./env/templates/template_index_article.php')) {
    $template = new NoSixTemplate('template_index_article.php', './env/templates');
  } else {
    $template = new NoSixTemplate('template_index_article.php', './templates');
  }
} else {
  if (file_exists('./env/templates/template_view.php')) {
    //ユーザーテンプレートがある場合はそちらを採用する
    $template = new NoSixTemplate('template_view.php', './env/templates/');
  } else {
    $template = new NoSixTemplate('template_view.php', './templates');
  }
}
$template->set_Data('caption', $caption);
$template->set_data('setting', $SETTING); //Array
$template->set_data('home_menu', $home_menu);
$template->set_data('accounts', $accounts); //Array[](url, icon)
$template->set_data('articles', $article); //Array[](id,title,created_at,comment,image)
$template->set_data('document', $document); //Array(max_index,min_index,current_index,article_count,search_words)
$template->set_data('login', $LOGIN);
$template->set_data('move_next', $next);
$template->set_data('move_prev', $prev);
$template->set_data('back_url', $BACK_URL);
$template->set_data('VERSION', $VERSION);
$template->set_Data('delete_link', $delete_link);
if ($static_mode == 1) {
  //staticモード
  $file_name = "./log_{$target}.html";
  ob_start();

  $template->display();

  $output = ob_get_contents();
  ob_end_clean();
  file_put_contents($file_name, $output);
  echo ('{"state":"success", "path", "' . $file_name . '"}');
  exit;
}
if ($rewrite_rel != "") {
  //rewriteモード
  ob_start();
  $template->display();
  $result  = ob_get_contents();
  ob_end_clean();
  $result = str_replace('"./', '"' . $rewrite_rel, $result);
  $result = str_replace("'./", "'" . $rewrite_rel, $result);
  echo $result;
} else {
  //通常表示モード
  $template->display();
}

//コメント内の特殊記述を変換するほか、コメントに対する特殊変換を行う
function convertCommands($comment, $image_rows, $index, $next, $prev)
{
  global $SETTING;
  global $VERSION;
  $comment = str_replace('{{', "&#060;&#060;", $comment); //{{ 表記をエスケープ
  $comment = str_replace('}}', "&#062;&#062;", $comment); //}} 表記をエスケープ

  //Markdown記述をParsedownでHTML変換
  $comment = preg_replace_callback(
    '/\{md\}(.*?)\{\/md\}/s',
    function ($matches) {
      require_once("./module/Parsedown.php");
      require_once("./module/ParsedownEx.php");

      $Parsedown = new ParsedownEx();
      return $Parsedown->text($matches[1]);
    },
    $comment
  );


  $novel_ctl_vt = "<span class='vt-controler-ctl'><i class='fa-solid fa-sliders' ></i></span><div class='vt-cover hide'></div><div class='vt-controler hide'><div><span class='vt-title-1'>文字サイズ</span><span class='vt-controler-1 vt-controler-active'>小</span><span class='vt-controler-2'>中</span><span class='vt-controler-3'>大</span></div><div><span class='vt-title-2'>文字組</span><span class='vt-controler-v vt-controler-active'>縦</span><span class='vt-controler-h'>横</span></div><div><span class='vt-title-3'>フォント</span><span class='vt-controler-fss vt-controler-active'>ゴシック体</span><span class='vt-controler-fs'>明朝体</span></div></div>";
  $novel_ctl_ht = "<span class='vt-controler-ctl'><i class='fa-solid fa-sliders' ></i></span><div class='vt-cover hide'></div><div class='vt-controler hide'><div><span class='vt-title-1'>文字サイズ</span><span class='vt-controler-1 vt-controler-active'>小</span><span class='vt-controler-2'>中</span><span class='vt-controler-3'>大</span></div><div><span class='vt-title-2'>文字組</span><span class='vt-controler-v'>縦</span><span class='vt-controler-h vt-controler-active'>横</span></div><div><span class='vt-title-3'>フォント</span><span class='vt-controler-fss vt-controler-active'>ゴシック体</span><span class='vt-controler-fs'>明朝体</span></div></div>";

  //単純置換
  $comment = preg_replace('/{search:(.*?)}/i', '<a href="./list.php?q=$1">$1</a>', $comment);
  $comment = preg_replace('/{tag:(.*?)}/i', '<a href="./list.php?q=%23$1" class="tag">#$1</a>', $comment);
  $comment = preg_replace('/{search (.*?)}/i', '<a href="./list.php?q=$1">$1</a>', $comment);
  $comment = preg_replace('/{tag (.*?)}/i', '<a href="./list.php?q=%23$1" class="tag">#$1</a>', $comment);

  $comment = preg_replace_callback('/\{ ?\.\.([^\}]+)\}/', function ($matches) {
    $values = explode(' ', $matches[1]);
    $class = 'user_' . implode(' user_', $values);
    return '<div class="' . $class . '">';
  }, $comment);
  $comment = preg_replace('/{\/\.\.}/i', "</div>", $comment);

  $comment = preg_replace_callback('/\{ ?\.([^\}]+)\}/', function ($matches) {
    $values = explode(' ', $matches[1]);
    $class = 'user_' . implode(' user_', $values);
    return '<span class="' . $class . '">';
  }, $comment);
  $comment = preg_replace('/{\/\.}/i', "</span>", $comment);

  $comment = preg_replace('/{alert}(.*?){\/alert}/s', "<div class='alert'><div>$1<div class='alert_control'><button class='alert_close button'>表示</button> <button class='historyback alert_return button_normal button '>戻る</button></div><div class='alert_control move_control alert_move_control'>$next | $prev</div></div></div>\r\n", $comment);
  $comment = preg_replace('/{more}(.*?){\/more}/s', "<div class='more-area hide'>$1</div><button data-label='続きを読む...' data-closelabel='閉じる' class='more-button more-button-active'>続きを読む...</button>\r\n", $comment);
  $comment = preg_replace('/{more (.*?)\|(.*?)}(.*?){\/more}/s', "<div class='more-area hide'>$3</div><button data-label='$1' data-closelabel='$2' class='more-button more-button-active'>$1</button>\r\n", $comment);
  $comment = preg_replace('/{# ?(.*?)}(.*?){\/#}/s', "<span style='color:#$1'>$2</span>", $comment);
  $comment = preg_replace('/{c}/i', "<div style='clear:both'></div>\r\n", $comment);
  $comment = preg_replace('/{tc}/i', "<span class='text-combine'>", $comment);
  $comment = preg_replace('/{\/tc}/i', "</span>", $comment);
  $comment = preg_replace('/{vt}/i', "$novel_ctl_vt<div class='novel-body-outer'><div class='novel-body vertical-text'><div class='novel-body-in'>", $comment);
  $comment = preg_replace('/{\/vt}/i', "</div></div></div>", $comment);
  $comment = preg_replace('/{ht}/i', "$novel_ctl_ht<div class='novel-body-outer'><div class='novel-body horizontal-text'><div class='novel-body-in'>", $comment);
  $comment = preg_replace('/{\/ht}/i', "</div></div></div>", $comment);
  $comment = preg_replace('/\{rb\}(.*?)\|(.*?)\{\/rb\}/', '<ruby>$1<rp>(</rp><rt>$2</rt><rp>)</rp></ruby>', $comment);
  $comment = preg_replace('/{border (.*?)}/i', "<style>.view-image{border:1px solid $1;}</style>", $comment);
  $comment = preg_replace('/{border}/i', "<style>.view-image{border:1px solid #555;}</style>", $comment);
  $comment = preg_replace('/{hidden:(.*?)}/i', "<style>#$1{display:none}</style>", $comment);
  $comment = preg_replace('/{hidden (.*?)}/i', "<style>#$1{display:none}</style>", $comment);
  $comment = preg_replace('/{mangaview}/i', "<script src=\"./script/manga_viewer.js?version=$VERSION'\"></script><link rel=\"stylesheet\" href=\"./style/manga_viewer.css?version=$VERSION\"><img class=\"hide manga-view-loader-src\" src=\"./resource/loading.png\">", $comment);
  $comment = preg_replace('/{mangaview r}/i', "<script>r_opening = true;</script><script src=\"./script/manga_viewer.js?version=$VERSION'\"></script><link rel=\"stylesheet\" href=\"./style/manga_viewer.css?version=$VERSION\"><img class=\"hide manga-view-loader-src\" src=\"./resource/loading.png\">", $comment);

  $comment = preg_replace_callback('/{mailform}(.*?){\/mailform}/s', function ($matches) {
    if (file_exists("./env/templates/template_mailform.php")) {
      include("./env/templates/template_mailform.php");
    } else {
      include("./templates/template_mailform.php");
    }
    $mailform_top = '<div class="mail_form" style="width:80%;margin:0 auto 0 auto"><form action="./api/mail_send.php" method="post" name="inputForm" id="inputForm">';
    $mailform_bottom = '<input type="text" name="title" style="display:none;"><div class="control" style="text-align:center"><input type="submit" value="送信" class="button" id="submit"></div></form></div>';
    return $mailform_top . $matches[1] . $mailform_bottom . $mailform_B;
  }, $comment);

  $comment = preg_replace_callback('/{mailform}/i', function ($matches) {
    if (file_exists("./env/templates/template_mailform.php")) {
      include("./env/templates/template_mailform.php");
    } else {
      include("./templates/template_mailform.php");
    }
    return $mailform_A . $mailform_B;
  }, $comment);

  //埋め込みyoutube対応
  $comment = preg_replace_callback('#<iframe([^>]*)src="([^"]*youtube.com[^"]*)"[^>]*>#i', function ($matches) {
    $attrs = $matches[1];
    $src = $matches[2];

    // class="youtube"がない場合は追加
    if (!preg_match('#\bclass=#i', $attrs)) {
      $attrs .= ' class="youtube"';
    } else {
      $attrs = preg_replace_callback('#\bclass="([^"]*)"#i', function ($classMatches) {
        if (strpos($classMatches[1], 'youtube') === false) {
          return 'class="' . $classMatches[1] . ' youtube"';
        } else {
          return $classMatches[0];
        }
      }, $attrs);
    }

    // iframeをラップ
    return '<div class="youtube_wrapper"><iframe' . $attrs . ' src="' . $src . '"></iframe></div>';
  }, $comment);

  //iOS向け特例処理
  $user_agent = $_SERVER['HTTP_USER_AGENT'];
  if (strpos($user_agent, 'iPhone') !== false || strpos($user_agent, 'iPad') !== false || strpos($user_agent, 'Macintosh') !== false) {
    $comment = preg_replace('/―/u', "─", $comment); //iOS safariのフォントの問題でダッシュがつながらないため罫線に変換
    $comment = preg_replace('/(─+)/u', "<span style='text-orientation:mixed;'>$1</span>", $comment); // iOS Safariのバグ対応 縦書きで―が横向きになる問題を修正
    $comment = preg_replace('/(…+)/u', "<span style='text-orientation:mixed;'>$1</span>", $comment); // iOS Safariのバグ対応 縦書きで…が横向きになる問題を修正
  }

  //画像ほか特殊記述の解釈実行
  $matches = array();
  preg_match_all('/{([^}]+)}/', $comment, $matches);
  $border = "";
  foreach ($matches[1] as $match) {
    $command_array = explode(" ", str_replace(":", " ", $match)); //{ ～ }の中身をスペースまたはコロン区切りで分割
    $firstCommand = array_shift($command_array); //先頭はコマンド
    if (is_numeric($firstCommand) && count($image_rows) >= $firstCommand) {
      //{1 b s} など先頭が数字の場合は画像表示( 数字は本記事添付画像のソート番号 )
      $img_comment = "";
      $target = "thumb_l";
      $file_name = h($image_rows[$firstCommand]["file_name"]);
      $type = h($image_rows[$firstCommand]["type"]);
      $container_s = '<div class="view-image-container">';
      $alt = "";
      foreach ($command_array as $command) {
        if (mb_substr($command, 0, 1) == '"') {
          $alt = 'alt="' . h(str_replace('"', "", $command)) . '"';
          continue;
        }
        switch ($command) {
          case "s":
            $target = 'thumb';
            break;
          case "l":
            $container_s = '<div class="view-image-left">';
            break;
          case "r":
            $container_s = '<div class="view-image-right">';
            break;
          case "c":
            $container_s = '<div class="view-image-container">';
            break;
          case "b":
            $border = " view-image-border";
            break;
          default:
            $img_comment .= $command;
            break;
        }
      }
      if ($img_comment == "") {
        $img_comment = $image_rows[$firstCommand]["comment"] ?? "";
      }

      $lightbox_link_a = "";
      $lightbox_link_b = "";
      if ($SETTING['use_lightbox']) {
        $lightbox_link_a = '<a class="lighbox-link" data-lightbox="images" href="./images/' . $image_rows[$firstCommand][$target . "_name"] . '">';
        $lightbox_link_b = '</a>';
      }
      if ($img_comment != "") {
        $img_comment = convertLineBreaks($img_comment);
        $img_comment = "<br><span class='image-comment'>" . $img_comment . "</span>";
      }

      if ($type == "video") {
        //typeがvideoの場合はvideoタグを生成
        $tag = $container_s . '<video controls class="view-image ' . $border . '" src="./images/' . $file_name  .
          '"  data-type="' . $type . '" data-src="./images/' .  $file_name  . '" width="' . $image_rows[$firstCommand][$target . "_width"]  . '" height="' . $file_name . '" ' . $alt .
          ' data-width="' . $image_rows[$firstCommand]["width"] . '" data-height="' . $image_rows[$firstCommand]["height"]  . '">' . $img_comment . '</div>';
      } else {
        //画像表示用の img タグ生成
        $tag = $container_s . '<a name="' . $firstCommand . '" id="' . $firstCommand . '"></a>' . $lightbox_link_a  . '<img class="view-image manga-view-page ' . $border . '" src="./images/' . $image_rows[$firstCommand][$target . "_name"]  .
          '"  data-type="' . $type . '" data-src="./images/' .  $file_name  . '" width="' . $image_rows[$firstCommand][$target . "_width"]  . '" height="' . $image_rows[$firstCommand][$target . "_height"] . '" ' . $alt .
          ' data-width="' . $image_rows[$firstCommand]["width"] . '" data-height="' . $image_rows[$firstCommand]["height"]  . '">' . $lightbox_link_b  . $img_comment . '</div>';
      }

      $comment = str_replace('{' . $match . '}', $tag, $comment);
    } else if ($firstCommand == "article_list") {
      //記事一覧 引数は タグ名（検索する文字）、 sort("desc","asc")、スタイル(image,text)、最大記事数
      //{article_list photos asc image} 等
      $arg_count = count($command_array);
      $search = $command_array[0];
      $arg_count >= 2  ? $sort = $command_array[1] : $sort = "asc";
      $arg_count >= 3  ? $style = $command_array[2] : $style = "";
      $arg_count >= 4  ? $records_per_page = $command_array[3] : $records_per_page  = 999;

      $tmp = get_article_list($search, $sort, $style, $records_per_page, $index);

      $comment = str_replace('{' . $match . '}', $tmp, $comment);
    } else {
      //解釈不能
      //$comment = str_replace('{' . $match . '}', '', $comment);
    }
  }
  $comment = str_replace("&#060;&#060;", '{',  $comment); //{{ 表記のエスケープを復元
  $comment = str_replace("&#062;&#062;", '}',  $comment); //}} 表記のエスケープを復元
  return $comment;
}

//記事一覧を生成する
function get_article_list($searchs, $order, $style, $records_per_page, $index)
{
  global $NSFW_CONSENT;
  global $target;
  global $db;
  global $SETTING;

  //並び替え順序設定
  $order = strtolower($order);
  if ($order == "like") {
    $order = "like desc, created_at desc";
  } else if ($order != "asc" && $order != "desc" && $order != "like") {
    $order = "created_at asc";
  } else {
    $order = "created_at " . $order;
  }


  if (!is_numeric(($index))) {
    $index = 1;
  }
  if (!is_numeric($records_per_page)) {
    $records_per_page = 9999;
  }
  //ページ指定時のオフセット位置を計算
  $offset = ($index - 1)  * $records_per_page;

  //一覧のスタイルを決定する
  $full_text = false;
  $use_plugin = false;
  if (preg_match('/^plugin_/i', $style)) {
    //ユーザースタイルプラグイン
    $use_plugin = true;
    if (file_exists("./env/plugin/$style.php")) {
      require_once("./env/plugin/$style.php");
    } else if (file_exists("./plugin/$style.php")) {
      require_once("./plugin/$style.php");
    } else {
      $list_html = "プラグインファイルが見つかりません： ./env/plugin/$style.php";
      return $list_html;
    }
    $plugin_func = "$style\\PluginArticleStyleFormat_Header";
    $list_html = $plugin_func(); //ヘッダーHTMLをプラグインから取得
  } else {
    switch ($style) {
      case "images": //画像一覧
        $list_html = "<div class='loglist_images'>";
        break;
      case "thumbs": //サムネイル一覧
        $list_html = "<ul class='loglist_thumbs'>";
        break;
      case "text": //テキストのみの一覧
        $list_html = "<ul class='loglist'>";
        break;
      case "full_text": //テキスト全体を表示する一覧
        $list_html = "<ul class='loglist'>";
        $full_text = true;
        $style = "text";
        break;
      default:
        //指定がない場合、不正な場合はテキスト一覧
        $list_html = "<ul class='loglist'>";
    }
  }

  try {
    //DBから記事一覧を取得
    $article_count = 0;

    if ($searchs == '' || $searchs == 'ALL') {
      //検索なし
      //hidden指定されていない記事を$records_per_page分取得する
      $article_count = $db->querySingle('SELECT Count(id) as count From articles WHERE hidden != 1 ');
      $pages = ceil($article_count / $records_per_page);
      $result = $db->query('SELECT * From articles  Where hidden != 1  ORDER BY ' . $order . ' LIMIT ' .  $records_per_page . ' OFFSET ' . $offset);
    } else {
      //検索条件あり
      //検索条件に従って記事を検索する。スペース区切りで複数指定があった場合はAND条件とする。
      //検索条件式の構築
      $search_list = '';
      $search_Idx = 0;
      foreach (explode('&&', $searchs)  as $search_Idx => $search) {
        $search_list = $search_list .  " and  tags like :search{$search_Idx}";
      }
      //検索結果数の取得
      $stmt = $db->prepare('SELECT Count(id) as count From articles WHERE hidden != 1 ' . $search_list);
      foreach (explode('&&', $searchs) as $search_Idx => $search) {
        $stmt->bindValue(":search{$search_Idx}",  '%' . $search . '%');
      }
      $result = $stmt->execute();
      $row = $result->fetchArray(); //結果は常に1行
      $article_count = $row['count'];
      $pages = ceil($article_count / $records_per_page);

      //記事の取得
      $stmt = $db->prepare('SELECT * From articles WHERE hidden != 1 ' . $search_list  .
        ' ORDER BY ' .  $order  .  ' LIMIT ' . $records_per_page . ' OFFSET ' . $offset);
      foreach (explode('&&', $searchs) as $search_Idx => $search) {
        $stmt->bindValue(":search{$search_Idx}",  '%' . $search . '%');
      }

      $result = $stmt->execute();
    }
    while ($row = $result->fetchArray()) {
      $id = h($row['id']);
      if ($id == $target) {
        //この記事自体は対象にしない
        continue;
      }

      $created_at = h($row['created_at']);
      $title = h($row['title']);
      $comment = $row['comment'];
      $nsfw = $row['nsfw'] ?? false;
      $thumbnail = $row['thumbnail'] ?? "";

      if ($SETTING['use_rewrite']) {
        if ($row['alt_url']) {
          $url = "./{$row['alt_url']}";
        } else {
          $url = "./log/$id";
        }
      } else {
        $url = "./view.php?id=$id";
      }
      //この記事のタグ一覧を生成
      $tags = "";
      foreach (explode(' ', $row['tags']) as $tag) {
        if ($tag != "") {
          if (substr($tag, 0, 1) == "-") {
            //1文字目が「-」のタグは非表示
            continue;
          }
          $tags = $tags . "<a href='./list.php?q=%23{$tag}'>#{$tag}</a> ";
        }
      }

      //記事内容の編集
      if ($comment) {
        preg_match('/{caption}(.*?){\/caption}/si', $comment, $match);
        if ($match) {
          //キャプションがある場合はそれを使用
          $comment = $match[1];
        } else {
          $comment = preg_replace('/\{.*\}/', '', $comment); //特殊記述の削除
          $comment = strip_tags($comment); //HTMLタグの削除

          // 一部マークダウンを削除する
          $comment = preg_replace('/^(#+)\s*(.*)/m', "$2", $comment);
          $comment = preg_replace('/\*\*(.*)\*\*/', "$1", $comment);
          $comment = preg_replace('/\*(.*)\*/', "$1", $comment);

          if (!$full_text) {
            //full_text指定がなければ100文字以降を省略し、改行を削除
            if (mb_strlen($comment) > 100) {
              $comment = mb_substr(strip_tags($comment), 0, 100) . " ... ";
            }
            $comment = convertLineBreaks($comment);
          }
        }
      }

      //スタイル別HTML生成
      if ($use_plugin) {
        $thumbs = $db->query('SELECT * FROM images where article_id = ' . $id . ' order by sort asc');
        $trow = $thumbs->fetchArray();
        $plugin_func = "$style\\PluginArticleStyleFormat_Body";
        $list_html .= $plugin_func(
          [
            'id' => $id,
            'created_at' => $created_at,
            'title' => $title,
            'comment' => $comment,
            'nsfw' => $nsfw,
            'tags' => $tags,
            'url' => $url,
          ],
          $trow
        );
      } else if ($style == "images") {
        //画像のみ
        if ($nsfw && !$NSFW_CONSENT) {
          $list_html = $list_html . "<div class='view-image-nsfw-container'><a href='{$url}'><div class='thumb-box background-nsfw'></div></a></div>";
        } else {
          $thumbs = $db->query('SELECT * FROM images where article_id = ' . $id . ' order by sort asc');
          if ($thumbnail == "@user_thumbnail") {
            //サムネイル用画像を読み捨て
            $trow = $thumbs->fetchArray();
          }
          $img = "";
          while ($trow = $thumbs->fetchArray()) {
            $thumb_file_name = h($trow["thumb_l_name"]);
            $img = '<div class="view-image-container"><img class="view-image" src="./images/' . $thumb_file_name  . '"' .
              ' data-src="./images/' .  h($trow["file_name"])  . '" width=' .  h($trow["thumb_l_width"]) . ' height=' .  h($trow["thumb_l_height"]) .
              ' data-width="' . h($trow["width"]) . '" data-height="' . h($trow["height"]) . '"></div>';
            $list_html = $list_html . $img;
          }
        }
      } else if ($style == "text") {
        //テキストのみ
        if ($nsfw && !$NSFW_CONSENT) {
          $title = '<span class="nsfw-titile">[NSWF]</span>' . $title;
        }
        $list_html = $list_html . "<li class='text'><div class='search-list-info'><span class='created_at'><a href='{$url}'>{$created_at}</a></span><span class='title'><a href='{$url}'>{$title}</a></span><div class='tags'>{$tags}</div><span class='comment'>{$comment}</span></div></a></li>";
      } else {
        //サムネ＋テキスト または サムネイルのみ
        if ($thumbnail == "@text") {
          //サムネイルなし
          $img = "<div class='list-thumb-text'><span class='list-thumb-text-title'>{$title}</span></div>";
          $list_html = $list_html . "<li><a href='{$url}'>{$img}</a><div class='search-list-info'><span class='created_at'><a href='{$url}'>{$created_at}</a></span><span class='title'><a href='{$url}'>{$title}</a></span><div class='tags'>{$tags}</div><span class='comment'>{$comment}</span></div></li>";
        } else if ($thumbnail && $thumbnail != "@user_thumbnail"  && $thumbnail != "@all") {
          //サムネイルファイル指定
          if ($nsfw && !$NSFW_CONSENT) {
            $img = "<div class='list-thumb-box  background-nsfw'\"></div>";
          } else {
            $img = "<div class='list-thumb-text' style=\"background-image:url('./env/thumbnails/{$thumbnail}')\"><span class='list-thumb-text-title'>{$title}</span></div>";
          }
          $list_html = $list_html . "<li><a href='{$url}'>{$img}</a><div class='search-list-info'><span class='created_at'><a href='{$url}'>{$created_at}</a></span><span class='title'><a href='{$url}'>{$title}</a></span><div class='tags'>{$tags}</div><span class='comment'>{$comment}</span></div></a></li>";
        } else {
          $thumbs = $db->query('SELECT * FROM images where article_id = ' . $id . ' order by sort asc');
          $img = "";
          if ($style == "thumbs" && ($thumbnail == "@all" || $SETTING["index_all_images"])) {
            //サムネイル一覧 かつ 全画像を表示する場合
            $tindex = 0;
            while ($trow = $thumbs->fetchArray()) {
              $tindex++;
              if ($nsfw && !$NSFW_CONSENT) {
                $img = "<div class='list-thumb-box  background-nsfw'\"></div>";
              } else {
                $file_name = h($trow['thumb_name']);
                $img = "<div class='list-thumb-box' style=\"background-image:url('./images/$file_name')\"></div>";
              }
              $list_html = $list_html . "<li><a href='{$url}#{$tindex}'>{$img}</a><div class='search-list-info'><span class='created_at'><a href='{$url}#{$tindex}'>{$created_at}</a></span><span class='title'><a href='{$url}#{$tindex}'>{$title}</a></span><div class='tags'>{$tags}</div><span class='comment'>{$comment}</span></div></li>";
            }
            if ($tindex == 0) {
              //全画像を表示する指定だが画像がない
              $img = "<div class='list-thumb-text'><span class='list-thumb-text-title'>{$title}</span></div>";
              $list_html = $list_html . "<li><a href='{$url}'>{$img}</a><div class='search-list-info'><span class='created_at'><a href='{$url}'>{$created_at}</a></span><span class='title'><a href='{$url}'>{$title}</a></span><div class='tags'>{$tags}</div><span class='comment'>{$comment}</span></div></li>";
            }
          } else {
            //通常スタイル
            if ($nsfw && !$NSFW_CONSENT) {
              $img = "<div class='list-thumb-box  background-nsfw'\"></div>";
            } else {
              $trow = $thumbs->fetchArray();
              if ($trow) {
                $file_name = h($trow['thumb_name']);
                $img = "<div class='list-thumb-box' style=\"background-image:url('./images/$file_name')\"></div>";
              } else {
                $img = "<div class='list-thumb-text'><span class='list-thumb-text-title'>{$title}</span></div>";
              }
            }
            $list_html = $list_html . "<li><a href='{$url}'>{$img}</a><div class='search-list-info'><span class='created_at'><a href='{$url}'>{$created_at}</a></span><span class='title'><a href='{$url}'>{$title}</a></span><div class='tags'>{$tags}</div><span class='comment'>{$comment}</span></div></li>";
          }
        }
      }
    }

    //ページインデックス
    $indexser = "<div class='pager'>";
    $max = $pages;
    if ($pages > 1) {
      if ($index < 5) {
        $min_page = 1;
        $max_page = $min_page + 9;
        if (
          $max_page > $max
        ) {
          $max_page = $max;
        }
      } else {
        $max_page = $index + 5;
        if (
          $max_page > $max
        ) {
          $max_page = $max;
        }
        $min_page = $max_page - 10;
      }
      if (
        $min_page < 1
      ) {
        $min_page = 1;
      }
      if (
        $max_page > $max
      ) {
        $max_page = $max;
      }
      for (
        $i = $min_page;
        $i <= $max_page;
        $i++
      ) {
        if ($i == $index) {
          $indexser = $indexser . '<span class="pager-current-index">' . $i . '</span> ';
        } else {
          if ($SETTING['use_rewrite']) {
            $indexser = $indexser . '<a href="./log/' . $target . '/' . $i . '">' . $i . '</a> ';
          } else {
            $indexser = $indexser . '<a href="./view.php?id=' . $target . '&page=' . $i . '">' . $i . '</a> ';
          }
        }
      }
    }
    $indexser .= "</div>";
    if ($use_plugin) {
      $plugin_func = "$style\\PluginArticleStyleFormat_Footer";
      $list_html .= $plugin_func();
    } else {
      $list_html .=  "</ul>";
    }
    return $list_html . $indexser;
  } catch (Exception $e) {
    echo $e->getMessage();
  }
}

//MarkDownによるテーブル表記をHTMLに変換
function markdownTableToHtml($text)
{
  // テーブルの行を抽出
  $pattern = '/\n\|(.*?)\|\n\|(.*?)\|\n((\|(.*?)\|\n)*)/im';
  $text = preg_replace_callback($pattern, function ($matches) {

    // ヘッダー行の処理
    $headerCells = explode('|', trim($matches[1]));
    $header = '<thead><tr>';
    foreach ($headerCells as $cell) {
      $header .= '<th>' . trim($cell) . '</th>';
    }
    $header .= '</tr></thead>';

    // データ行の処理
    $rows = explode("\n", trim($matches[3]));
    $body = '<tbody>';
    foreach ($rows as $row) {
      $body .= '<tr>';
      $cells = explode('|', trim(substr($row, 1, strlen($row) - 2)));
      foreach ($cells as $cell) {
        $body .= '<td>' . trim($cell) . '</td>';
      }
      $body .= '</tr>';
    }
    $body .= '</tbody>';

    // HTMLテーブルを作成
    $table = '<table class="md">' . $header . $body . '</table>';
    return $table;
  }, $text);

  return $text;
}
